"""Generated message classes for cloudasset version v1p4alpha1.

The cloud asset API manages the history and inventory of cloud resources.
"""
# NOTE: This file is autogenerated and should not be edited by hand.


from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding


package = 'cloudasset'


class AnalyzeIamPolicyResponse(_messages.Message):
  r"""A response message for AssetService.AnalyzeIamPolicy.

  Fields:
    analysisResults: A list of IamPolicyAnalysisResult that matches the
      request, or empty if no result is found.
    fullyExplored: Represents whether all entries in the analysis_results have
      been fully explored to answer the query in the request.
    nonCriticalErrors: A list of non-critical errors happened during the
      request handling to explain why `fully_explored` is false, or empty if
      no error happened.
  """

  analysisResults = _messages.MessageField('IamPolicyAnalysisResult', 1, repeated=True)
  fullyExplored = _messages.BooleanField(2)
  nonCriticalErrors = _messages.MessageField('GoogleCloudAssetV1p4alpha1AnalysisState', 3, repeated=True)


class Binding(_messages.Message):
  r"""Associates `members` with a `role`.

  Fields:
    condition: The condition that is associated with this binding. NOTE: An
      unsatisfied condition will not allow user access via current binding.
      Different bindings, including their conditions, are examined
      independently.
    members: Specifies the identities requesting access for a Cloud Platform
      resource. `members` can have the following values:  * `allUsers`: A
      special identifier that represents anyone who is    on the internet;
      with or without a Google account.  * `allAuthenticatedUsers`: A special
      identifier that represents anyone    who is authenticated with a Google
      account or a service account.  * `user:{emailid}`: An email address that
      represents a specific Google    account. For example,
      `alice@example.com` .   * `serviceAccount:{emailid}`: An email address
      that represents a service    account. For example, `my-other-
      app@appspot.gserviceaccount.com`.  * `group:{emailid}`: An email address
      that represents a Google group.    For example, `admins@example.com`.  *
      `deleted:user:{emailid}?uid={uniqueid}`: An email address (plus unique
      identifier) representing a user that has been recently deleted. For
      example,`alice@example.com?uid=123456789012345678901`. If the user is
      recovered, this value reverts to `user:{emailid}` and the recovered user
      retains the role in the binding.  *
      `deleted:serviceAccount:{emailid}?uid={uniqueid}`: An email address
      (plus    unique identifier) representing a service account that has been
      recently    deleted. For example,    `my-other-
      app@appspot.gserviceaccount.com?uid=123456789012345678901`.    If the
      service account is undeleted, this value reverts to
      `serviceAccount:{emailid}` and the undeleted service account retains the
      role in the binding.  * `deleted:group:{emailid}?uid={uniqueid}`: An
      email address (plus unique    identifier) representing a Google group
      that has been recently    deleted. For example,
      `admins@example.com?uid=123456789012345678901`. If    the group is
      recovered, this value reverts to `group:{emailid}` and the    recovered
      group retains the role in the binding.   * `domain:{domain}`: The G
      Suite domain (primary) that represents all the    users of that domain.
      For example, `google.com` or `example.com`.
    role: Role that is assigned to `members`. For example, `roles/viewer`,
      `roles/editor`, or `roles/owner`.
  """

  condition = _messages.MessageField('Expr', 1)
  members = _messages.StringField(2, repeated=True)
  role = _messages.StringField(3)


class CloudassetAnalyzeIamPolicyRequest(_messages.Message):
  r"""A CloudassetAnalyzeIamPolicyRequest object.

  Fields:
    accessSelector_permissions: Optional. The permissions to appear in result.
    accessSelector_roles: Optional. The roles to appear in result.
    identitySelector_identity: Required. The identity appear in the form of
      members in [IAM policy
      binding](https://cloud.google.com/iam/reference/rest/v1/Binding).
    options_expandGroups: Optional. If true, the identities section of the
      result will expand any Google groups appearing in an IAM policy binding.
      If identity_selector is specified, the identity in the result will be
      determined by the selector, and this flag will have no effect.  Default
      is false.
    options_expandResources: Optional. If true, the resource section of the
      result will expand any resource attached to an IAM policy to include
      resources lower in the resource hierarchy.  For example, if the request
      analyzes for which resources user A has permission P, and the results
      include an IAM policy with P on a GCP folder, the results will also
      include resources in that folder with permission P.  If
      resource_selector is specified, the resource section of the result will
      be determined by the selector, and this flag will have no effect.
      Default is false.
    options_expandRoles: Optional. If true, the access section of result will
      expand any roles appearing in IAM policy bindings to include their
      permissions.  If access_selector is specified, the access section of the
      result will be determined by the selector, and this flag will have no
      effect.  Default is false.
    options_maxFanoutsPerGroup: Optional. The maximum number of fanouts per
      group when expand_groups is enabled.
    options_maxFanoutsPerResource: Optional. The maximum number of fanouts per
      parent resource, such as GCP Project etc., when expand_resources is
      enabled.
    options_outputGroupEdges: Optional. If true, the result will output group
      identity edges, starting from the binding's group members, to any
      expanded identities. Default is false.
    options_outputPartialResultBeforeTimeout: Optional. If true, you will get
      a response with partial result instead of a DEADLINE_EXCEEDED error when
      your request processing takes longer than the deadline.
    options_outputResourceEdges: Optional. If true, the result will output
      resource edges, starting from the policy attached resource, to any
      expanded resources. Default is false.
    parent: Required. The relative name of root asset to do analysis . This
      can only be an organization number (such as "organizations/123") for
      now.
    resourceSelector_fullResourceName: Required. The [full resource name](http
      s://cloud.google.com/apis/design/resource_names#full_resource_name) .
  """

  accessSelector_permissions = _messages.StringField(1, repeated=True)
  accessSelector_roles = _messages.StringField(2, repeated=True)
  identitySelector_identity = _messages.StringField(3)
  options_expandGroups = _messages.BooleanField(4)
  options_expandResources = _messages.BooleanField(5)
  options_expandRoles = _messages.BooleanField(6)
  options_maxFanoutsPerGroup = _messages.IntegerField(7, variant=_messages.Variant.INT32)
  options_maxFanoutsPerResource = _messages.IntegerField(8, variant=_messages.Variant.INT32)
  options_outputGroupEdges = _messages.BooleanField(9)
  options_outputPartialResultBeforeTimeout = _messages.BooleanField(10)
  options_outputResourceEdges = _messages.BooleanField(11)
  parent = _messages.StringField(12, required=True)
  resourceSelector_fullResourceName = _messages.StringField(13)


class Expr(_messages.Message):
  r"""Represents an expression text. Example:      title: "User account
  presence"     description: "Determines whether the request has a user
  account"     expression: "size(request.user) > 0"

  Fields:
    description: An optional description of the expression. This is a longer
      text which describes the expression, e.g. when hovered over it in a UI.
    expression: Textual representation of an expression in Common Expression
      Language syntax.  The application context of the containing message
      determines which well-known feature set of CEL is supported.
    location: An optional string indicating the location of the expression for
      error reporting, e.g. a file name and a position in the file.
    title: An optional title for the expression, i.e. a short string
      describing its purpose. This can be used e.g. in UIs which allow to
      enter the expression.
  """

  description = _messages.StringField(1)
  expression = _messages.StringField(2)
  location = _messages.StringField(3)
  title = _messages.StringField(4)


class GoogleCloudAssetV1p4alpha1Access(_messages.Message):
  r"""A role or permission that appears in an access control list.

  Fields:
    analysisState: The analysis state of this access node.
    permission: The permission.
    role: The role.
  """

  analysisState = _messages.MessageField('GoogleCloudAssetV1p4alpha1AnalysisState', 1)
  permission = _messages.StringField(2)
  role = _messages.StringField(3)


class GoogleCloudAssetV1p4alpha1AccessControlList(_messages.Message):
  r"""An access control list, derived from the above IAM policy binding, which
  contains a set of resources and accesses. May include one item from each set
  to compose an access control entry.  NOTICE that there could be multiple
  access control lists for one IAM policy binding. The access control lists
  are created per resource type.  For example, assume we have the following
  cases in one IAM policy binding: - Permission P1 and P2 apply to resource R1
  and R2 of resource type RT1; - Permission P3 applies to resource R3 and R4
  of resource type RT2;  This will result in the following access control
  lists: - AccessControlList 1: RT1, [R1, R2], [P1, P2] - AccessControlList 2:
  RT2, [R3, R4], [P3]

  Fields:
    accesses: The accesses that match one of the following conditions: - The
      access_selector, if it is specified in request; - Otherwise, access
      specifiers reachable from the policy binding's role.
    baseResourceType: The unified resource type name of the resource type that
      this access control list is based on, such as
      "compute.googleapis.com/Instance" for Compute Engine instance, etc.
    resourceEdges: Resource edges of the graph starting from the policy
      attached resource to any descendant resources. The Edge.source_node
      contains the full resource name of a parent resource and
      Edge.target_node contains the full resource name of a child resource.
      This field is present only if the output_resource_edges option is
      enabled in request.
    resources: The resources that match one of the following conditions: - The
      resource_selector, if it is specified in request; - Otherwise, resources
      reachable from the policy attached resource.
  """

  accesses = _messages.MessageField('GoogleCloudAssetV1p4alpha1Access', 1, repeated=True)
  baseResourceType = _messages.StringField(2)
  resourceEdges = _messages.MessageField('GoogleCloudAssetV1p4alpha1Edge', 3, repeated=True)
  resources = _messages.MessageField('GoogleCloudAssetV1p4alpha1Resource', 4, repeated=True)


class GoogleCloudAssetV1p4alpha1AnalysisState(_messages.Message):
  r"""Represents analysis state of each node in the result graph or non-
  critical errors in the response.

  Enums:
    CodeValueValuesEnum: The Google standard error code that best describes
      the state. For example: - OK means the node has been successfully
      explored; - PERMISSION_DENIED means an access denied error is
      encountered; - DEADLINE_EXCEEDED means the node hasn't been explored in
      time;

  Fields:
    cause: The human-readable description of the cause of failure.
    code: The Google standard error code that best describes the state. For
      example: - OK means the node has been successfully explored; -
      PERMISSION_DENIED means an access denied error is encountered; -
      DEADLINE_EXCEEDED means the node hasn't been explored in time;
  """

  class CodeValueValuesEnum(_messages.Enum):
    r"""The Google standard error code that best describes the state. For
    example: - OK means the node has been successfully explored; -
    PERMISSION_DENIED means an access denied error is encountered; -
    DEADLINE_EXCEEDED means the node hasn't been explored in time;

    Values:
      OK: Not an error; returned on success  HTTP Mapping: 200 OK
      CANCELLED: The operation was cancelled, typically by the caller.  HTTP
        Mapping: 499 Client Closed Request
      UNKNOWN: Unknown error.  For example, this error may be returned when a
        `Status` value received from another address space belongs to an error
        space that is not known in this address space.  Also errors raised by
        APIs that do not return enough error information may be converted to
        this error.  HTTP Mapping: 500 Internal Server Error
      INVALID_ARGUMENT: The client specified an invalid argument.  Note that
        this differs from `FAILED_PRECONDITION`.  `INVALID_ARGUMENT` indicates
        arguments that are problematic regardless of the state of the system
        (e.g., a malformed file name).  HTTP Mapping: 400 Bad Request
      DEADLINE_EXCEEDED: The deadline expired before the operation could
        complete. For operations that change the state of the system, this
        error may be returned even if the operation has completed
        successfully.  For example, a successful response from a server could
        have been delayed long enough for the deadline to expire.  HTTP
        Mapping: 504 Gateway Timeout
      NOT_FOUND: Some requested entity (e.g., file or directory) was not
        found.  Note to server developers: if a request is denied for an
        entire class of users, such as gradual feature rollout or undocumented
        whitelist, `NOT_FOUND` may be used. If a request is denied for some
        users within a class of users, such as user-based access control,
        `PERMISSION_DENIED` must be used.  HTTP Mapping: 404 Not Found
      ALREADY_EXISTS: The entity that a client attempted to create (e.g., file
        or directory) already exists.  HTTP Mapping: 409 Conflict
      PERMISSION_DENIED: The caller does not have permission to execute the
        specified operation. `PERMISSION_DENIED` must not be used for
        rejections caused by exhausting some resource (use
        `RESOURCE_EXHAUSTED` instead for those errors). `PERMISSION_DENIED`
        must not be used if the caller can not be identified (use
        `UNAUTHENTICATED` instead for those errors). This error code does not
        imply the request is valid or the requested entity exists or satisfies
        other pre-conditions.  HTTP Mapping: 403 Forbidden
      UNAUTHENTICATED: The request does not have valid authentication
        credentials for the operation.  HTTP Mapping: 401 Unauthorized
      RESOURCE_EXHAUSTED: Some resource has been exhausted, perhaps a per-user
        quota, or perhaps the entire file system is out of space.  HTTP
        Mapping: 429 Too Many Requests
      FAILED_PRECONDITION: The operation was rejected because the system is
        not in a state required for the operation's execution.  For example,
        the directory to be deleted is non-empty, an rmdir operation is
        applied to a non-directory, etc.  Service implementors can use the
        following guidelines to decide between `FAILED_PRECONDITION`,
        `ABORTED`, and `UNAVAILABLE`:  (a) Use `UNAVAILABLE` if the client can
        retry just the failing call.  (b) Use `ABORTED` if the client should
        retry at a higher level      (e.g., when a client-specified test-and-
        set fails, indicating the      client should restart a read-modify-
        write sequence).  (c) Use `FAILED_PRECONDITION` if the client should
        not retry until      the system state has been explicitly fixed.
        E.g., if an "rmdir"      fails because the directory is non-empty,
        `FAILED_PRECONDITION`      should be returned since the client should
        not retry unless      the files are deleted from the directory.  HTTP
        Mapping: 400 Bad Request
      ABORTED: The operation was aborted, typically due to a concurrency issue
        such as a sequencer check failure or transaction abort.  See the
        guidelines above for deciding between `FAILED_PRECONDITION`,
        `ABORTED`, and `UNAVAILABLE`.  HTTP Mapping: 409 Conflict
      OUT_OF_RANGE: The operation was attempted past the valid range.  E.g.,
        seeking or reading past end-of-file.  Unlike `INVALID_ARGUMENT`, this
        error indicates a problem that may be fixed if the system state
        changes. For example, a 32-bit file system will generate
        `INVALID_ARGUMENT` if asked to read at an offset that is not in the
        range [0,2^32-1], but it will generate `OUT_OF_RANGE` if asked to read
        from an offset past the current file size.  There is a fair bit of
        overlap between `FAILED_PRECONDITION` and `OUT_OF_RANGE`.  We
        recommend using `OUT_OF_RANGE` (the more specific error) when it
        applies so that callers who are iterating through a space can easily
        look for an `OUT_OF_RANGE` error to detect when they are done.  HTTP
        Mapping: 400 Bad Request
      UNIMPLEMENTED: The operation is not implemented or is not
        supported/enabled in this service.  HTTP Mapping: 501 Not Implemented
      INTERNAL: Internal errors.  This means that some invariants expected by
        the underlying system have been broken.  This error code is reserved
        for serious errors.  HTTP Mapping: 500 Internal Server Error
      UNAVAILABLE: The service is currently unavailable.  This is most likely
        a transient condition, which can be corrected by retrying with a
        backoff. Note that it is not always safe to retry non-idempotent
        operations.  See the guidelines above for deciding between
        `FAILED_PRECONDITION`, `ABORTED`, and `UNAVAILABLE`.  HTTP Mapping:
        503 Service Unavailable
      DATA_LOSS: Unrecoverable data loss or corruption.  HTTP Mapping: 500
        Internal Server Error
    """
    OK = 0
    CANCELLED = 1
    UNKNOWN = 2
    INVALID_ARGUMENT = 3
    DEADLINE_EXCEEDED = 4
    NOT_FOUND = 5
    ALREADY_EXISTS = 6
    PERMISSION_DENIED = 7
    UNAUTHENTICATED = 8
    RESOURCE_EXHAUSTED = 9
    FAILED_PRECONDITION = 10
    ABORTED = 11
    OUT_OF_RANGE = 12
    UNIMPLEMENTED = 13
    INTERNAL = 14
    UNAVAILABLE = 15
    DATA_LOSS = 16

  cause = _messages.StringField(1)
  code = _messages.EnumField('CodeValueValuesEnum', 2)


class GoogleCloudAssetV1p4alpha1Edge(_messages.Message):
  r"""A directional edge.

  Fields:
    sourceNode: The source node of the edge.
    targetNode: The target node of the edge.
  """

  sourceNode = _messages.StringField(1)
  targetNode = _messages.StringField(2)


class GoogleCloudAssetV1p4alpha1Identity(_messages.Message):
  r"""An identity that appears in an access control list.

  Fields:
    analysisState: The analysis state of this identity node.
    name: The identity name in any form of members appear in [IAM policy
      binding](https://cloud.google.com/iam/reference/rest/v1/Binding), such
      as: - user:foo@google.com - group:group1@google.com -
      serviceAccount:s1@prj1.iam.gserviceaccount.com -
      projectOwner:some_project_id - domain:google.com - allUsers - etc.
  """

  analysisState = _messages.MessageField('GoogleCloudAssetV1p4alpha1AnalysisState', 1)
  name = _messages.StringField(2)


class GoogleCloudAssetV1p4alpha1IdentityList(_messages.Message):
  r"""A GoogleCloudAssetV1p4alpha1IdentityList object.

  Fields:
    groupEdges: Group identity edges of the graph starting from the binding's
      group members to any node of the identities. The Edge.source_node
      contains a group, such as "group:parent@google.com". The
      Edge.target_node contains a member of the group, such as
      "group:child@google.com" or "user:foo@google.com". This field is present
      only if the output_group_edges option is enabled in request.
    identities: Only the identities that match one of the following conditions
      will be presented: - The identity_selector, if it is specified in
      request; - Otherwise, identities reachable from the policy binding's
      members.
  """

  groupEdges = _messages.MessageField('GoogleCloudAssetV1p4alpha1Edge', 1, repeated=True)
  identities = _messages.MessageField('GoogleCloudAssetV1p4alpha1Identity', 2, repeated=True)


class GoogleCloudAssetV1p4alpha1Resource(_messages.Message):
  r"""A GCP resource that appears in an access control list.

  Fields:
    analysisState: The analysis state of this resource node.
    fullResourceName: The [full resource name](https://aip.dev/122#full-
      resource-names).
  """

  analysisState = _messages.MessageField('GoogleCloudAssetV1p4alpha1AnalysisState', 1)
  fullResourceName = _messages.StringField(2)


class IamPolicyAnalysisResult(_messages.Message):
  r"""IAM Policy analysis result, consisting of one IAM policy binding and
  derived access control lists.

  Fields:
    accessControlLists: The access control lists derived from the iam_binding
      that match or potentially match resource and access selectors specified
      in the request.
    attachedResourceFullName: The full name of the resource to which the
      iam_binding policy attaches.
    fullyExplored: Represents whether all nodes in the transitive closure of
      the iam_binding node have been explored.
    iamBinding: The Cloud IAM policy binding under analysis.
    identityList: The identity list derived from members of the iam_binding
      that match or potentially match identity selector specified in the
      request.
  """

  accessControlLists = _messages.MessageField('GoogleCloudAssetV1p4alpha1AccessControlList', 1, repeated=True)
  attachedResourceFullName = _messages.StringField(2)
  fullyExplored = _messages.BooleanField(3)
  iamBinding = _messages.MessageField('Binding', 4)
  identityList = _messages.MessageField('GoogleCloudAssetV1p4alpha1IdentityList', 5)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')
encoding.AddCustomJsonFieldMapping(
    CloudassetAnalyzeIamPolicyRequest, 'accessSelector_permissions', 'accessSelector.permissions')
encoding.AddCustomJsonFieldMapping(
    CloudassetAnalyzeIamPolicyRequest, 'accessSelector_roles', 'accessSelector.roles')
encoding.AddCustomJsonFieldMapping(
    CloudassetAnalyzeIamPolicyRequest, 'identitySelector_identity', 'identitySelector.identity')
encoding.AddCustomJsonFieldMapping(
    CloudassetAnalyzeIamPolicyRequest, 'options_expandGroups', 'options.expandGroups')
encoding.AddCustomJsonFieldMapping(
    CloudassetAnalyzeIamPolicyRequest, 'options_expandResources', 'options.expandResources')
encoding.AddCustomJsonFieldMapping(
    CloudassetAnalyzeIamPolicyRequest, 'options_expandRoles', 'options.expandRoles')
encoding.AddCustomJsonFieldMapping(
    CloudassetAnalyzeIamPolicyRequest, 'options_maxFanoutsPerGroup', 'options.maxFanoutsPerGroup')
encoding.AddCustomJsonFieldMapping(
    CloudassetAnalyzeIamPolicyRequest, 'options_maxFanoutsPerResource', 'options.maxFanoutsPerResource')
encoding.AddCustomJsonFieldMapping(
    CloudassetAnalyzeIamPolicyRequest, 'options_outputGroupEdges', 'options.outputGroupEdges')
encoding.AddCustomJsonFieldMapping(
    CloudassetAnalyzeIamPolicyRequest, 'options_outputPartialResultBeforeTimeout', 'options.outputPartialResultBeforeTimeout')
encoding.AddCustomJsonFieldMapping(
    CloudassetAnalyzeIamPolicyRequest, 'options_outputResourceEdges', 'options.outputResourceEdges')
encoding.AddCustomJsonFieldMapping(
    CloudassetAnalyzeIamPolicyRequest, 'resourceSelector_fullResourceName', 'resourceSelector.fullResourceName')



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Cloud Pub/Sub message transforms test command."""

from googlecloudsdk.api_lib.pubsub import message_transforms
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.pubsub import flags
from googlecloudsdk.command_lib.pubsub import util
from googlecloudsdk.core.util import http_encoding


def _Run(args):
  """Runs the message transforms test command."""
  client = message_transforms.MessageTransformsClient()

  message_body = getattr(args, 'message', None)
  attributes = util.ParseAttributes(
      getattr(args, 'attribute', None), client.messages
  )
  message_transforms_file = getattr(args, 'message_transforms_file', None)
  topic = getattr(args, 'topic', None)
  if topic:
    topic = args.CONCEPTS.topic.Parse()
  subscription = getattr(args, 'subscription', None)
  if subscription:
    subscription = args.CONCEPTS.subscription.Parse()

  result = client.Test(
      project_ref=util.ParseProject(),
      message_body=http_encoding.Encode(message_body),
      attributes=attributes,
      message_transforms_file=message_transforms_file,
      topic_ref=topic,
      subscription_ref=subscription,
  )
  output = []
  for transformed_message in result.transformedMessages:
    if message := transformed_message.transformedMessage:
      message_copy = {}
      for field in message.all_fields():
        value = getattr(message, field.name)
        if value:
          if field.name == 'data':
            value = message.data.decode()
          message_copy[field.name] = value
      output.append(message_copy)
    else:
      output.append(transformed_message)
  return output


@base.DefaultUniverseOnly
@base.ReleaseTracks(
    base.ReleaseTrack.GA, base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA
)
class Test(base.Command):
  """Tests message transforms against a given message."""

  @staticmethod
  def Args(parser):
    flags.AddTestMessageTransformFlags(parser)

  def Run(self, args):
    return _Run(args)



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Support library for managing deployments."""


import os
import re
import sys
import uuid
import zipfile

from apitools.base.py import encoding
from apitools.base.py import exceptions as apitools_exceptions
from apitools.base.py import extra_types
from googlecloudsdk.api_lib.infra_manager import configmanager_util
from googlecloudsdk.api_lib.storage import storage_api
from googlecloudsdk.api_lib.storage import storage_util
from googlecloudsdk.calliope import exceptions as c_exceptions
from googlecloudsdk.command_lib.infra_manager import deterministic_snapshot
from googlecloudsdk.command_lib.infra_manager import errors
from googlecloudsdk.command_lib.infra_manager import staging_bucket_util
from googlecloudsdk.core import log
from googlecloudsdk.core import requests
from googlecloudsdk.core import resources
from googlecloudsdk.core.resource import resource_transform
from googlecloudsdk.core.util import files
from googlecloudsdk.core.util import times
import six


if sys.version_info >= (3, 6):
  # pylint: disable=g-import-not-at-top
  from googlecloudsdk.command_lib.infra_manager import tfvars_parser


def _UploadSourceDirToGCS(gcs_client, source, gcs_source_staging, ignore_file):
  """Uploads a local directory to GCS as a zip archive.

  Uploads a zip archive of the directory for compatibility with the back-end.

  Args:
    gcs_client: a storage_api.StorageClient instance for interacting with GCS.
    source: string, a path to a local directory.
    gcs_source_staging: resources.Resource, the bucket to upload to. This must
      already exist.
    ignore_file: optional string, a path to a gcloudignore file.

  Returns:
    The generation number of the uploaded GCS object.
  """

  source_snapshot = deterministic_snapshot.DeterministicSnapshot(
      source, ignore_file=ignore_file
  )

  size_str = resource_transform.TransformSize(source_snapshot.uncompressed_size)
  log.status.Print(
      'Uploading {num_files} file(s) totalling {size}.'.format(
          num_files=len(source_snapshot.files), size=size_str
      )
  )

  with files.TemporaryDirectory() as tmp_dir:
    zip_path = os.path.join(six.ensure_text(tmp_dir), 'source.zip')
    with zipfile.ZipFile(zip_path, 'w', zipfile.ZIP_DEFLATED) as zip_file:
      for file_metadata in source_snapshot.GetSortedFiles():
        full_local_path = os.path.join(file_metadata.root, file_metadata.path)
        zip_info = zipfile.ZipInfo(
            file_metadata.path, date_time=(1980, 1, 1, 0, 0, 0)
        )
        zip_info.compress_type = zipfile.ZIP_DEFLATED
        zip_file.writestr(
            zip_info, files.ReadBinaryFileContents(full_local_path)
        )

    target_obj_ref = resources.REGISTRY.Parse(
        'gs://{0}/{1}'.format(
            gcs_source_staging.bucket, gcs_source_staging.object
        ),
        collection='storage.objects',
    )
    return gcs_client.CopyFileToGCS(zip_path, target_obj_ref).generation


def _UploadSourceToGCS(
    source, stage_bucket, deployment_short_name, location, ignore_file
):
  """Uploads local content to GCS.

  This will ensure that the source and destination exist before triggering the
  upload.

  Args:
    source: string, a local path.
    stage_bucket: optional string. When not provided, the default staging bucket
      will be used (see GetDefaultStagingBucket). This string is of the format
      "gs://bucket-name/". An "im_source_staging" object will be created under
      this bucket, and any uploaded artifacts will be stored there.
    deployment_short_name: short name of the deployment.
    location: location of the deployment.
    ignore_file: string, a path to a gcloudignore file.

  Returns:
    A string in the format "gs://path/to/resulting/upload#generation".

  Raises:
    RequiredArgumentException: if stage-bucket is owned by another project.
    BadFileException: if the source doesn't exist or isn't a directory.
  """
  gcs_client = storage_api.StorageClient()

  if stage_bucket is None:
    used_default_bucket_name = True
    gcs_source_staging_dir = staging_bucket_util.DefaultGCSStagingDir(
        deployment_short_name, location
    )
  else:
    used_default_bucket_name = False
    gcs_source_staging_dir = '{0}{1}/{2}/{3}'.format(
        stage_bucket,
        staging_bucket_util.STAGING_DIR,
        location,
        deployment_short_name,
    )

  # By calling REGISTRY.Parse on "gs://my-bucket/foo", the result's "bucket"
  # property will be "my-bucket" and the "object" property will be "foo".
  gcs_source_staging_dir_ref = resources.REGISTRY.Parse(
      gcs_source_staging_dir, collection='storage.objects'
  )

  # Make sure the bucket exists
  try:
    gcs_client.CreateBucketIfNotExists(
        gcs_source_staging_dir_ref.bucket,
        check_ownership=used_default_bucket_name,
    )
  except storage_api.BucketInWrongProjectError:
    raise c_exceptions.RequiredArgumentException(
        'stage-bucket',
        'A bucket with name {} already exists and is owned by '
        'another project. Specify a bucket using '
        '--stage-bucket.'.format(gcs_source_staging_dir_ref.bucket),
    )

  # This will look something like this:
  # "1615850562.234312-044e784992744951b0cd71c0b011edce.zip"
  staged_object = '{stamp}-{uuid}.zip'.format(
      stamp=times.GetTimeStampFromDateTime(times.Now()),
      uuid=uuid.uuid4().hex,
  )

  if gcs_source_staging_dir_ref.object:
    staged_object = gcs_source_staging_dir_ref.object + '/' + staged_object

  gcs_source_staging = resources.REGISTRY.Create(
      collection='storage.objects',
      bucket=gcs_source_staging_dir_ref.bucket,
      object=staged_object,
  )

  if not os.path.exists(source):
    raise c_exceptions.BadFileException(
        'could not find source [{}]'.format(source)
    )

  if not os.path.isdir(source):
    raise c_exceptions.BadFileException(
        'source is not a directory [{}]'.format(source)
    )

  generation = _UploadSourceDirToGCS(
      gcs_client, source, gcs_source_staging, ignore_file
  )

  upload_bucket = 'gs://{0}/{1}'.format(
      gcs_source_staging.bucket, gcs_source_staging.object
  )

  return '{0}#{1}'.format(upload_bucket, generation)


def UpdateDeploymentDeleteRequestWithForce(unused_ref, unused_args, request):
  """UpdateDeploymentDeleteRequestWithForce adds force flag to delete request."""

  request.force = True
  return request


def DeleteCleanupStagedObjects(response, unused_args):
  """DeploymentDeleteCleanupStagedObjects deletes staging gcs objects created as part of deployment apply command."""

  # do not delete staged objects if delete results in error
  if response.error is not None:
    return
  if response.metadata is not None:
    md = encoding.MessageToPyValue(response.metadata)
    # entity could be deployment, previews
    entity_full_name = md['target']
    entity_id = entity_full_name.split('/')[5]
    location = entity_full_name.split('/')[3]
    staging_gcs_directory = staging_bucket_util.DefaultGCSStagingDir(
        entity_id, location
    )
    gcs_client = storage_api.StorageClient()
    staging_bucket_util.DeleteStagingGCSFolder(
        gcs_client, staging_gcs_directory
    )
  return response


def _CreateTFBlueprint(
    messages,
    deployment_short_name,
    preview_short_name,
    location,
    local_source,
    stage_bucket,
    ignore_file,
    gcs_source,
    git_source_repo,
    git_source_directory,
    git_source_ref,
    input_values,
):
  """Returns the TerraformBlueprint message.

  Args:
    messages: ModuleType, the messages module that lets us form Config API
      messages based on our protos.
    deployment_short_name: short name of the deployment.
    preview_short_name: short name of the preview.
    location: location of the deployment.
    local_source: Local storage path where config files are stored.
    stage_bucket: optional string. Destination for storing local config files
      specified by local source flag. e.g. "gs://bucket-name/".
    ignore_file: optional string, a path to a gcloudignore file.
    gcs_source:  URI of an object in Google Cloud Storage. e.g.
      `gs://{bucket}/{object}`
    git_source_repo: Repository URL.
    git_source_directory: Subdirectory inside the git repository.
    git_source_ref: Git branch or tag.
    input_values: Input variable values for the Terraform blueprint. It only
      accepts (key, value) pairs where value is a scalar value.

  Returns:
    A messages.TerraformBlueprint to use with deployment operation.
  """

  terraform_blueprint = messages.TerraformBlueprint(
      inputValues=input_values,
  )

  if gcs_source is not None:
    terraform_blueprint.gcsSource = gcs_source
  elif local_source is not None and deployment_short_name is not None:
    upload_bucket = _UploadSourceToGCS(
        local_source, stage_bucket, deployment_short_name, location, ignore_file
    )
    terraform_blueprint.gcsSource = upload_bucket
  elif local_source is not None and preview_short_name is not None:
    upload_bucket = _UploadSourceToGCS(
        local_source, stage_bucket, preview_short_name, location, ignore_file
    )
    terraform_blueprint.gcsSource = upload_bucket
  else:
    terraform_blueprint.gitSource = messages.GitSource(
        repo=git_source_repo,
        directory=git_source_directory,
        ref=git_source_ref,
    )

  return terraform_blueprint


def _CreateProviderConfig(
    messages,
    provider_source,
):
  """Returns the ProviderConfig message.

  Args:
    messages: ModuleType, the messages module that lets us form Config API
      messages based on our protos.
    provider_source: Input to control from where to fetch providers.

  Returns:
      A messages.ProviderConfig to use with deployment operation or None if
      provider_source is not set.
  """
  if provider_source is None:
    return None
  provider_config = messages.ProviderConfig(
      sourceType=messages.ProviderConfig.SourceTypeValueValuesEnum(
          provider_source
      )
  )
  return provider_config


def Apply(
    messages,
    async_,
    deployment_full_name,
    service_account,
    tf_version_constraint=None,
    local_source=None,
    stage_bucket=None,
    ignore_file=None,
    import_existing_resources=False,
    artifacts_gcs_bucket=None,
    worker_pool=None,
    gcs_source=None,
    git_source_repo=None,
    git_source_directory=None,
    git_source_ref=None,
    input_values=None,
    inputs_file=None,
    labels=None,
    quota_validation=None,
    annotations=None,
    provider_source=None,
):
  """Updates the deployment if one exists, otherwise creates a deployment.

  Args:
    messages: ModuleType, the messages module that lets us form blueprints API
      messages based on our protos.
    async_: bool, if True, gcloud will return immediately, otherwise it will
      wait on the long-running operation.
    deployment_full_name: string, the fully qualified name of the deployment,
      e.g. "projects/p/locations/l/deployments/d".
    service_account: User-specified Service Account (SA) to be used as
      credential to manage resources. e.g.
      `projects/{projectID}/serviceAccounts/{serviceAccount}` The default Cloud
      Build SA will be used initially if this field is not set.
    tf_version_constraint: User-specified Terraform version constraint, for
      example, "=1.3.10".
    local_source: Local storage path where config files are stored.
    stage_bucket: optional string. Destination for storing local config files
      specified by local source flag. e.g. "gs://bucket-name/".
    ignore_file: optional string, a path to a gcloudignore file.
    import_existing_resources: By default, Infrastructure Manager will return a
      failure when Terraform encounters a 409 code (resource conflict error)
      during actuation. If this flag is set to true, Infrastructure Manager will
      instead attempt to automatically import the resource into the Terraform
      state (for supported resource types) and continue actuation.
    artifacts_gcs_bucket: User-defined location of Cloud Build logs and
      artifacts in Google Cloud Storage. e.g. `gs://{bucket}/{folder}` A default
      bucket will be bootstrapped if the field is not set or empty
    worker_pool: The User-specified Worker Pool resource in which the Cloud
      Build job will execute. If this field is unspecified, the default Cloud
      Build worker pool will be used. e.g.
      projects/{project}/locations/{location}/workerPools/{workerPoolId}
    gcs_source:  URI of an object in Google Cloud Storage. e.g.
      `gs://{bucket}/{object}`
    git_source_repo: Repository URL.
    git_source_directory: Subdirectory inside the git repository.
    git_source_ref: Git branch or tag.
    input_values: Input variable values for the Terraform blueprint. It only
      accepts (key, value) pairs where value is a scalar value.
    inputs_file: Accepts .tfvars file.
    labels: User-defined metadata for the deployment.
    quota_validation: Input to control quota checks for resources in terraform'
      configuration files.
    annotations: User-defined annotations for the deployment.
    provider_source: Input to control from where to fetch providers.

  Returns:
    The resulting Deployment resource or, in the case that async_ is True, a
      long-running operation.

  Raises:
    InvalidArgumentException: If an invalid set of flags is provided (e.g.
      trying to run with --target-git-subdir but without --target-git).
  """

  labels_message, annotations_message = _GeneratesLabelsAnnotations(
      messages.Deployment(), labels, annotations
  )

  additional_properties = []
  if input_values is not None:
    for key, value in six.iteritems(input_values):
      additional_properties.append(
          messages.TerraformBlueprint.InputValuesValue.AdditionalProperty(
              key=key,
              value=messages.TerraformVariable(
                  inputValue=encoding.PyValueToMessage(
                      extra_types.JsonValue, value
                  )
              ),
          )
      )
  elif inputs_file is not None:
    if sys.version_info < (3, 6):
      raise errors.InvalidDataError(
          '--inputs-file flag is only supported for python version 3.6 and'
          ' above.'
      )
    tfvar_values = tfvars_parser.ParseTFvarFile(inputs_file)
    for key, value in six.iteritems(tfvar_values):
      additional_properties.append(
          messages.TerraformBlueprint.InputValuesValue.AdditionalProperty(
              key=key,
              value=messages.TerraformVariable(
                  inputValue=encoding.PyValueToMessage(
                      extra_types.JsonValue, value
                  )
              ),
          )
      )

  tf_input_values = messages.TerraformBlueprint.InputValuesValue(
      additionalProperties=additional_properties
  )

  deployment_ref = resources.REGISTRY.Parse(
      deployment_full_name, collection='config.projects.locations.deployments'
  )
  # Get just the ID from the fully qualified name.
  deployment_id = deployment_ref.Name()

  location = deployment_ref.Parent().Name()
  project = deployment_ref.Parent().Parent().Name()
  # Check if a deployment with the given name already exists. If it does, we'll
  # update that deployment. If not, we'll create it.
  try:
    existing_deployment = configmanager_util.GetDeployment(deployment_full_name)
  except apitools_exceptions.HttpNotFoundError:
    existing_deployment = None

  if existing_deployment is not None:
    # cleanup objects before uploading any new staging object
    _CleanupGCSStagingObjectsNotInUse(
        location, deployment_full_name, deployment_id
    )

  tf_blueprint = _CreateTFBlueprint(
      messages,
      deployment_id,
      None,
      location,
      local_source,
      stage_bucket,
      ignore_file,
      gcs_source,
      git_source_repo,
      git_source_directory,
      git_source_ref,
      tf_input_values,
  )
  provider_config = _CreateProviderConfig(messages, provider_source)

  deployment = messages.Deployment(
      name=deployment_full_name,
      serviceAccount=service_account,
      importExistingResources=import_existing_resources,
      workerPool=worker_pool,
      terraformBlueprint=tf_blueprint,
      labels=labels_message,
      annotations=annotations_message,
      tfVersionConstraint=tf_version_constraint,
      quotaValidation=quota_validation,
      providerConfig=provider_config,
  )

  if artifacts_gcs_bucket is not None:
    deployment.artifactsGcsBucket = artifacts_gcs_bucket

  is_creating_deployment = existing_deployment is None
  op = None

  if deployment.tfVersionConstraint is not None:
    tf_version = _GetTerraformVersionFromTfConstraint(
        deployment.tfVersionConstraint
    )
  elif (
      existing_deployment is not None
      and existing_deployment.tfVersionConstraint is not None
  ):
    tf_version = _GetTerraformVersionFromTfConstraint(
        existing_deployment.tfVersionConstraint
    )
  else:
    tf_version = None

  if tf_version is not None:
    tf_version_full_name = (
        f'projects/{project}/locations/{location}/terraformVersions/{tf_version}'
    )
    _DisplayObsoleteTfVersionMessage(messages, tf_version_full_name, tf_version)

  if is_creating_deployment:
    op = _CreateDeploymentOp(deployment, deployment_full_name)
  else:
    op = _UpdateDeploymentOp(
        deployment, existing_deployment, deployment_full_name, labels
    )

  log.debug('LRO: %s', op.name)

  # If the user chose to run asynchronously, then we'll match the output that
  # the automatically-generated Delete command issues and return immediately.
  if async_:
    log.status.Print(
        '{0} request issued for: [{1}]'.format(
            'Create' if is_creating_deployment else 'Update', deployment_id
        )
    )

    log.status.Print('Check operation [{}] for status.'.format(op.name))

    return op

  progress_message = '{} the deployment'.format(
      'Creating' if is_creating_deployment else 'Updating'
  )

  applied_deployment = configmanager_util.WaitForApplyDeploymentOperation(
      op, progress_message
  )

  if (
      applied_deployment.state
      == messages.Deployment.StateValueValuesEnum.FAILED
  ):
    raise errors.OperationFailedError(applied_deployment.stateDetail)

  return applied_deployment


def _CreateDeploymentOp(
    deployment,
    deployment_full_name,
):
  """Initiates and returns a CreateDeployment operation.

  Args:
    deployment: A partially filled messages.Deployment. The deployment will be
      filled with other details before the operation is initiated.
    deployment_full_name: string, the fully qualified name of the deployment,
      e.g. "projects/p/locations/l/deployments/d".

  Returns:
    The CreateDeployment operation.
  """
  deployment_ref = resources.REGISTRY.Parse(
      deployment_full_name, collection='config.projects.locations.deployments'
  )
  location_ref = deployment_ref.Parent()
  # Get just the ID from the fully qualified name.
  deployment_id = deployment_ref.Name()

  log.info('Creating the deployment')
  return configmanager_util.CreateDeployment(
      deployment, deployment_id, location_ref.RelativeName()
  )


def _UpdateDeploymentOp(
    deployment,
    existing_deployment,
    deployment_full_name,
    labels,
):
  """Initiates and returns an UpdateDeployment operation.

  Args:
    deployment: A partially filled messages.Deployment. The deployment will be
      filled with its target (e.g. configController, gitTarget, etc.) before the
      operation is initiated.
    existing_deployment: A messages.Deployment. The existing deployment to
      update.
    deployment_full_name: string, the fully qualified name of the deployment,
      e.g. "projects/p/locations/l/deployments/d".
    labels: dictionary of string → string, labels to be associated with the
      deployment.

  Returns:
    The UpdateDeployment operation.
  """

  log.info('Updating the existing deployment')

  # If the user didn't specify labels here, then we don't want to overwrite
  # the existing labels on the deployment, so we provide them back to the
  # underlying API.
  if labels is None:
    deployment.labels = existing_deployment.labels

  return configmanager_util.UpdateDeployment(deployment, deployment_full_name)


def ImportStateFile(messages, deployment_full_name, lock_id, file=None):
  """Creates a signed uri to upload the state file.

  Args:
    messages: ModuleType, the messages module that lets us form Infra Manager
      API messages based on our protos.
    deployment_full_name: string, the fully qualified name of the deployment,
      e.g. "projects/p/locations/l/deployments/d".
    lock_id: Lock ID of the lock file to verify person importing owns lock.
    file: file path of the statefile to be uploaded.

  Returns:
    A messages.StateFile which contains signed uri to be used to upload a state
    file.
  """

  import_state_file_request = messages.ImportStatefileRequest(
      lockId=int(lock_id),
  )

  log.status.Print('Creating signed uri for ImportStatefile request...')

  state_file = configmanager_util.ImportStateFile(
      import_state_file_request, deployment_full_name
  )
  if file is None:
    return state_file
  state_file_url = state_file.signedUri
  with files.BinaryFileReader(os.path.abspath(file)) as state_file_obj:
    requests.GetSession().put(state_file_url, data=state_file_obj)

  log.status.Print(f'Statefile {file} uploaded successfully.')
  return


def ExportDeploymentStateFile(
    messages, deployment_full_name, draft=False, file=None
    ):
  """Creates a signed uri to download the state file.

  Args:
    messages: ModuleType, the messages module that lets us form Infra Manager
      API messages based on our protos.
    deployment_full_name: string, the fully qualified name of the deployment,
      e.g. "projects/p/locations/l/deployments/d".
    draft: Lock ID of the lock file to verify person importing owns lock.
    file: string, the file name to download statefile to.

  Returns:
    A messages.StateFile which contains signed uri to be used to download a
    state file.
  """

  export_deployment_state_file_request = (
      messages.ExportDeploymentStatefileRequest(
          draft=draft,
      )
  )

  log.status.Print('Initiating export state file request...')

  state_file = configmanager_util.ExportDeploymentStateFile(
      export_deployment_state_file_request, deployment_full_name
  )
  if file is None:
    return state_file
  state_file_data = requests.GetSession().get(
      state_file.signedUri
      )
  if file.endswith(os.sep):
    file += 'statefile'
  files.WriteBinaryFileContents(file +'.tfstate', state_file_data.content)
  log.status.Print(
      f'Exported statefile {file}.tfstate'
      )
  return


def ExportRevisionStateFile(messages, deployment_full_name):
  """Creates a signed uri to download the state file.

  Args:
    messages: ModuleType, the messages module that lets us form Infra Manager
      API messages based on our protos.
    deployment_full_name: string, the fully qualified name of the deployment,
      e.g. "projects/p/locations/l/deployments/d".

  Returns:
    A messages.StateFile which contains signed uri to be used to upload a state
    file.
  """

  export_revision_state_file_request = messages.ExportRevisionStatefileRequest()

  log.status.Print('Initiating export state file request...')

  state_file = configmanager_util.ExportRevisionStateFile(
      export_revision_state_file_request, deployment_full_name
  )

  return state_file


def ExportLock(deployment_full_name):
  """Exports lock info of the deployment.

  Args:
    deployment_full_name: string, the fully qualified name of the deployment,
      e.g. "projects/p/locations/l/deployments/d".

  Returns:
    A lock info response.
  """

  log.status.Print('Initiating export lock request...')

  lock_info = configmanager_util.ExportLock(
      deployment_full_name,
  )

  return lock_info


def LockDeployment(messages, async_, deployment_full_name):
  """Locks the deployment.

  Args:
    messages: ModuleType, the messages module that lets us form Infra Manager
      API messages based on our protos.
    async_: bool, if True, gcloud will return immediately, otherwise it will
      wait on the long-running operation.
    deployment_full_name: string, the fully qualified name of the deployment,
      e.g. "projects/p/locations/l/deployments/d".

  Returns:
    A lock info resource or, in case async_ is True, a
      long-running operation.
  """

  lock_deployment_request = messages.LockDeploymentRequest()

  op = configmanager_util.LockDeployment(
      lock_deployment_request, deployment_full_name
  )

  deployment_ref = resources.REGISTRY.Parse(
      deployment_full_name, collection='config.projects.locations.deployments'
  )
  # Get just the ID from the fully qualified name.
  deployment_id = deployment_ref.Name()

  log.debug('LRO: %s', op.name)

  if async_:
    log.status.Print(
        'Lock deployment request issued for: [{0}]'.format(deployment_id)
    )

    log.status.Print('Check operation [{}] for status.'.format(op.name))
    return op

  progress_message = 'Locking the deployment'

  lock_response = configmanager_util.WaitForApplyDeploymentOperation(
      op, progress_message
  )

  if (
      lock_response.lockState
      == messages.Deployment.LockStateValueValuesEnum.LOCK_FAILED
  ):
    raise errors.OperationFailedError('Lock deployment operation failed.')

  return ExportLock(deployment_full_name)


def UnlockDeployment(
    messages,
    async_,
    deployment_full_name,
    lock_id,
):
  """Unlocks the deployment.

  Args:
    messages: ModuleType, the messages module that lets us form Infra Manager
      API messages based on our protos.
    async_: bool, if True, gcloud will return immediately, otherwise it will
      wait on the long-running operation.
    deployment_full_name: string, the fully qualified name of the deployment,
      e.g. "projects/p/locations/l/deployments/d".
    lock_id: Lock ID of the deployment to be unlocked.

  Returns:
    A deployment resource or, in case async_ is True, a
      long-running operation.
  """

  unlock_deployment_request = messages.UnlockDeploymentRequest(
      lockId=int(lock_id),
  )

  deployment_ref = resources.REGISTRY.Parse(
      deployment_full_name, collection='config.projects.locations.deployments'
  )
  # Get just the ID from the fully qualified name.
  deployment_id = deployment_ref.Name()

  op = configmanager_util.UnlockDeployment(
      unlock_deployment_request, deployment_full_name
  )

  log.debug('LRO: %s', op.name)

  if async_:
    log.status.Print(
        'Unlock deployment request issued for: [{0}]'.format(deployment_id)
    )

    log.status.Print('Check operation [{}] for status.'.format(op.name))

    return op

  progress_message = 'Unlocking the deployment'

  unlock_response = configmanager_util.WaitForApplyDeploymentOperation(
      op, progress_message
  )

  if (
      unlock_response.lockState
      == messages.Deployment.LockStateValueValuesEnum.UNLOCK_FAILED
  ):
    raise errors.OperationFailedError('Unlock deployment operation failed.')

  return unlock_response


def _CleanupGCSStagingObjectsNotInUse(
    location, deployment_full_name, deployment_id
):
  """Deletes staging object for all revisions except for last successful revision.

  Args:
    location: The location of deployment.
    deployment_full_name: string, the fully qualified name of the deployment,
      e.g. "projects/p/locations/l/deployments/d".
    deployment_id: the short name of the deployment.

  Raises:
    NotFoundError: If the bucket or folder does not exist.
  """

  gcs_client = storage_api.StorageClient()
  gcs_staging_dir = staging_bucket_util.DefaultGCSStagingDir(
      deployment_id, location
  )
  gcs_staging_dir_ref = resources.REGISTRY.Parse(
      gcs_staging_dir, collection='storage.objects'
  )
  bucket_ref = storage_util.BucketReference(gcs_staging_dir_ref.bucket)

  staged_objects = set()
  try:
    items = gcs_client.ListBucket(bucket_ref, gcs_staging_dir_ref.object)
    for item in items:
      item_dir = '/'.join(item.name.split('/')[:4])
      staged_objects.add(
          'gs://{0}/{1}'.format(gcs_staging_dir_ref.bucket, item_dir)
      )
    if not staged_objects:
      return
  except storage_api.BucketNotFoundError:
    # if staging bucket does not exist, there is nothing to clean.
    return

  op = configmanager_util.ListRevisions(deployment_full_name)
  revisions = sorted(
      op.revisions, key=lambda x: GetRevisionNumber(x.name), reverse=True
  )

  # discard gcsSource in latest revision from being deleted
  lastest_revision = revisions[0]
  if lastest_revision.terraformBlueprint is not None:
    staged_objects.discard(lastest_revision.terraformBlueprint.gcsSource)

  # discard staged object for last successful revision from being deleted
  for revision in revisions:
    if str(revision.state) == 'APPLIED':
      if revision.terraformBlueprint is not None:
        staged_objects.discard(revision.terraformBlueprint.gcsSource)
      break

  for obj in staged_objects:
    staging_bucket_util.DeleteStagingGCSFolder(gcs_client, obj)


def GetRevisionNumber(revision_full_name):
  """Returns the revision number from the revision name.

     e.g. - returns 12 for
     projects/p1/locations/l1/deployments/d1/revisions/r-12.

  Args:
    revision_full_name: string, the fully qualified name of the revision, e.g.
      "projects/p/locations/l/deployments/d/revisions/r-12".

  Returns:
    a revision number.
  """
  revision_ref = resources.REGISTRY.Parse(
      revision_full_name,
      collection='config.projects.locations.deployments.revisions',
  )
  revision_short_name = revision_ref.Name()
  return int(revision_short_name[2:])


def ExportPreviewResult(messages, preview_full_name, file=None):
  """Creates a signed uri to download the preview results.

  Args:
    messages: ModuleType, the messages module that lets us form Infra Manager
      API messages based on our protos.
    preview_full_name: string, the fully qualified name of the preview,
      e.g. "projects/p/locations/l/previews/p".]
    file: string, the file name to download results to.
  Returns:
    A messages.ExportPreviewResultResponse which contains signed uri to
    download binary and json plan files.
  """

  export_preview_result_request = (
      messages.ExportPreviewResultRequest()
  )

  log.status.Print('Initiating export preview results...')

  preview_result_resp = configmanager_util.ExportPreviewResult(
      export_preview_result_request, preview_full_name
  )
  if file is None:
    return preview_result_resp
  binary_data = requests.GetSession().get(
      preview_result_resp.result.binarySignedUri
      )
  json_data = requests.GetSession().get(
      preview_result_resp.result.jsonSignedUri
      )
  if file.endswith(os.sep):
    file += 'preview'
  files.WriteBinaryFileContents(file +'.tfplan', binary_data.content)
  files.WriteBinaryFileContents(file + '.json', json_data.content)
  log.status.Print(
      f'Exported preview artifacts {file}.tfplan and {file}.json'
      )
  return


def Create(
    messages,
    async_,
    preview_full_name,
    location_full_name,
    deployment,
    preview_mode,
    service_account,
    local_source=None,
    stage_bucket=None,
    ignore_file=None,
    artifacts_gcs_bucket=None,
    worker_pool=None,
    gcs_source=None,
    git_source_repo=None,
    git_source_directory=None,
    git_source_ref=None,
    input_values=None,
    inputs_file=None,
    labels=None,
    annotations=None,
    provider_source=None,
    tf_version_constraint=None,
):
  """Creates a preview.

  Args:
    messages: ModuleType, the messages module that lets us form blueprints API
      messages based on our protos.
    async_: bool, if True, gcloud will return immediately, otherwise it will
      wait on the long-running operation.
    preview_full_name: string, full path name of the preview. e.g.
      "projects/p/locations/l/previews/preview".
    location_full_name: string, full path name of the location. e.g.
      "projects/p/locations/l".
    deployment: deployment reference for a preview.
    preview_mode: preview mode to be set for a preview.
    service_account: User-specified Service Account (SA) to be used as
      credential to manage resources. e.g.
      `projects/{projectID}/serviceAccounts/{serviceAccount}` The default Cloud
      Build SA will be used initially if this field is not set.
    local_source: Local storage path where config files are stored.
    stage_bucket: optional string. Destination for storing local config files
      specified by local source flag. e.g. "gs://bucket-name/".
    ignore_file: optional string, a path to a gcloudignore file.
    artifacts_gcs_bucket: User-defined location of Cloud Build logs and
      artifacts in Google Cloud Storage. e.g. `gs://{bucket}/{folder}` A default
      bucket will be bootstrapped if the field is not set or empty
    worker_pool: The User-specified Worker Pool resource in which the Cloud
      Build job will execute. If this field is unspecified, the default Cloud
      Build worker pool will be used. e.g.
      projects/{project}/locations/{location}/workerPools/{workerPoolId}
    gcs_source:  URI of an object in Google Cloud Storage. e.g.
      `gs://{bucket}/{object}`
    git_source_repo: Repository URL.
    git_source_directory: Subdirectory inside the git repository.
    git_source_ref: Git branch or tag.
    input_values: Input variable values for the Terraform blueprint. It only
      accepts (key, value) pairs where value is a scalar value.
    inputs_file: Accepts .tfvars file.
    labels: User-defined metadata for the preview.
    annotations: User-defined annotations for the preview.
    provider_source: Input to control where to fetch providers.
    tf_version_constraint: User-specified Terraform version constraint, for
      example, "=1.3.10".

  Returns:
    The resulting Preview resource or, in the case that async_ is True, a
      long-running operation.

  Raises:
    InvalidArgumentException: If an invalid set of flags is provided (e.g.
      trying to run with --target-git-subdir but without --target-git).
  """

  additional_properties = _ParseInputValuesAndFile(
      input_values, inputs_file, messages)
  labels_message, annotations_message = _GeneratesLabelsAnnotations(
      messages.Preview(), labels, annotations
  )

  tf_input_values = messages.TerraformBlueprint.InputValuesValue(
      additionalProperties=additional_properties
  )

  if preview_full_name is not None:
    preview_ref = resources.REGISTRY.Parse(
        preview_full_name, collection='config.projects.locations.previews'
    )
    location = preview_ref.Parent().Name()
    preview_id = preview_ref.Name()
  else:
    location_ref = resources.REGISTRY.Parse(
        location_full_name, collection='config.projects.locations'
    )
    location = location_ref.Name()
    # Hardcoding preview_id when preview_full_name is none as it's needed for
    # bucket creation for local source in the CreateTFBlueprint workflow.
    preview_id = 'im-preview-{uuid}'.format(uuid=uuid.uuid4())

  tf_blueprint = _CreateTFBlueprint(
      messages,
      None,
      preview_id,
      location,
      local_source,
      stage_bucket,
      ignore_file,
      gcs_source,
      git_source_repo,
      git_source_directory,
      git_source_ref,
      tf_input_values,
  )
  provider_config = _CreateProviderConfig(messages, provider_source)

  preview = messages.Preview(
      serviceAccount=service_account,
      workerPool=worker_pool,
      labels=labels_message,
      annotations=annotations_message,
      providerConfig=provider_config,
      tfVersionConstraint=tf_version_constraint,
  )

  # set tf_blueprint only when one of the three sources is specified.
  if any(_ is not None for _ in (gcs_source, local_source, git_source_repo)):
    preview.terraformBlueprint = tf_blueprint

  if preview_full_name is not None:
    preview.name = preview_full_name

  if preview_mode is not None:
    try:
      preview.previewMode = messages.Preview.PreviewModeValueValuesEnum(
          preview_mode
      )
    except TypeError:
      raise c_exceptions.UnknownArgumentException(
          '--preview-mode',
          'Invalid preview mode. Supported values are DEFAULT and DELETE.')

  if deployment is not None:
    preview.deployment = deployment

  if artifacts_gcs_bucket is not None:
    preview.artifactsGcsBucket = artifacts_gcs_bucket

  op = _CreatePreviewOp(preview, preview_full_name, location_full_name)

  log.debug('LRO: %s', op.name)

  # If the user chose to run asynchronously, then we'll match the output that
  # the automatically-generated Delete command issues and return immediately.
  if async_:
    log.status.Print(
        '{0} request issued for: [{1}]'.format(
            'Create', preview_id
        )
    )

    log.status.Print('Check operation [{}] for status.'.format(op.name))

    return op

  progress_message = '{} the preview'.format(
      'Creating'
  )

  applied_preview = configmanager_util.WaitForCreatePreviewOperation(
      op, progress_message
  )

  if (
      applied_preview.state
      == messages.Preview.StateValueValuesEnum.FAILED
  ):
    raise errors.OperationFailedError(applied_preview.errorStatus.message)

  log.status.Print('Created preview: {}'.format(applied_preview.name))
  return applied_preview


def _CreatePreviewOp(preview, preview_full_name, location_full_name):
  """Initiates and returns a CreatePreview operation.

  Args:
    preview: A partially filled messages.preview. The preview will be filled
      with other details before the operation is initiated.
    preview_full_name: string, the fully qualified name of the preview, e.g.
      "projects/p/locations/l/previews/p".
    location_full_name: string, the fully qualified name of the location, e.g.
      "projects/p/locations/l".

  Returns:
    The CreatePreview operation.
  """
  if preview_full_name is None:
    return configmanager_util.CreatePreview(preview, None, location_full_name)
  preview_ref = resources.REGISTRY.Parse(
      preview_full_name, collection='config.projects.locations.previews')
  preview_id = preview_ref.Name()
  log.info('Creating the preview')
  return configmanager_util.CreatePreview(
      preview, preview_id, location_full_name
  )


def _ParseInputValuesAndFile(input_values, inputs_file, messages):
  """Parses input file or values and returns a list of additional properties.

  Args:
    input_values: Input variable values for the Terraform blueprint. It only
      accepts (key, value) pairs where value is a scalar value.
    inputs_file: Accepts .tfvars file.
    messages: ModuleType, the messages module that lets us form blueprints API
      messages based on our protos.

  Returns:
    The additional_properties list.
  """
  additional_properties = []
  if input_values is not None:
    for key, value in six.iteritems(input_values):
      additional_properties.append(
          messages.TerraformBlueprint.InputValuesValue.AdditionalProperty(
              key=key,
              value=messages.TerraformVariable(
                  inputValue=encoding.PyValueToMessage(
                      extra_types.JsonValue, value
                  )
              ),
          )
      )
  elif inputs_file is not None:
    tfvar_values = tfvars_parser.ParseTFvarFile(inputs_file)
    for key, value in six.iteritems(tfvar_values):
      additional_properties.append(
          messages.TerraformBlueprint.InputValuesValue.AdditionalProperty(
              key=key,
              value=messages.TerraformVariable(
                  inputValue=encoding.PyValueToMessage(
                      extra_types.JsonValue, value
                  )
              ),
          )
      )
  return additional_properties


def _GeneratesLabelsAnnotations(resource, labels=None, annotations=None):
  """Generates labels and annotations messages.

  Args:
    resource: Resource type, can be deployment or preview.
    labels: dict[str,str], labels to be associated with the resource.
    annotations: dict[str,str], annotations to be associated with the resource.

  Returns:
    Label and annotation messages.
  """
  labels_message = {}
  annotations_message = {}
  # Whichever labels the user provides will become the full set of labels in the
  # resulting deployment or preview.
  if labels is not None:
    labels_message = resource.LabelsValue(
        additionalProperties=[
            resource.LabelsValue.AdditionalProperty(key=key, value=value)
            for key, value in six.iteritems(labels)
        ]
    )

  if annotations is not None:
    annotations_message = resource.AnnotationsValue(
        additionalProperties=[
            resource.AnnotationsValue.AdditionalProperty(key=key, value=value)
            for key, value in six.iteritems(annotations)
        ]
    )
  return labels_message, annotations_message


def _GetTerraformVersionName(parent, tf_version):
  """Returns the Terraform version name.
  """
  return f'{parent}/terraformVersions/{tf_version}'


def _GetTerraformVersionFromTfConstraint(tf_version_constraint):
  """Returns the Terraform version from the Terraform version constraint.
  """
  m = re.match(
      r'^=?\s*(\d+\.\d+\.\d+)$',
      tf_version_constraint,
  )
  if m:
    return m.group(1)
  return None


def _DisplayObsoleteTfVersionMessage(
    messages, tf_version_full_name, tf_version
):
  """Checks if TF version is obsolete/deprecated and displays a warning."""
  try:
    tf_version_obj = configmanager_util.GetTfVersion(tf_version_full_name)
    if tf_version_obj.state in [
        messages.TerraformVersion.StateValueValuesEnum.DEPRECATED,
        messages.TerraformVersion.StateValueValuesEnum.OBSOLETE,
    ]:
      log.warning(
          'Terraform version %s is %s.', tf_version, tf_version_obj.state
      )
  except apitools_exceptions.HttpError:
    pass



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Vertex AI indexes update command."""


from apitools.base.py import encoding
from googlecloudsdk.api_lib.ai import operations
from googlecloudsdk.api_lib.ai.indexes import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.ai import constants
from googlecloudsdk.command_lib.ai import endpoint_util
from googlecloudsdk.command_lib.ai import flags
from googlecloudsdk.command_lib.ai import indexes_util
from googlecloudsdk.command_lib.ai import operations_util
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.core import log


@base.ReleaseTracks(base.ReleaseTrack.GA)
class UpdateV1(base.UpdateCommand):
  """Update an Vertex AI index.

  ## EXAMPLES

  To update index `123` under project `example` in region `us-central1`, run:

    $ {command} --display-name=new-name
    --metadata-file=path/to/your/metadata.json --project=example
    --region=us-central1
  """

  @staticmethod
  def Args(parser):
    flags.AddIndexResourceArg(parser, 'to update')
    flags.GetDisplayNameArg('index', required=False).AddToParser(parser)
    flags.GetDescriptionArg('index').AddToParser(parser)
    flags.GetMetadataFilePathArg('index').AddToParser(parser)
    labels_util.AddUpdateLabelsFlags(parser)

  def _Run(self, args, version):
    index_ref = args.CONCEPTS.index.Parse()
    region = index_ref.AsDict()['locationsId']
    project_id = index_ref.AsDict()['projectsId']
    with endpoint_util.AiplatformEndpointOverrides(version, region=region):
      index_client = client.IndexesClient(version=version)
      if version == constants.GA_VERSION:
        operation = index_client.Patch(index_ref, args)
      else:
        operation = index_client.PatchBeta(index_ref, args)
      if args.metadata_file is not None:
        # Update index content.
        op_ref = indexes_util.ParseIndexOperation(operation.name)
        log.status.Print(
            constants.OPERATION_CREATION_DISPLAY_MESSAGE.format(
                name=operation.name,
                verb='update index',
                id=op_ref.Name(),
                sub_commands='--index={} --region={} [--project={}]'.format(
                    index_ref.Name(), region, project_id)))
        # We will not wait for the operation since it can take up to hours.
        return operation

      # Update index meta data.
      response_msg = operations_util.WaitForOpMaybe(
          operations_client=operations.OperationsClient(version=version),
          op=operation,
          op_ref=indexes_util.ParseIndexOperation(operation.name),
          log_method='update')
      if response_msg is not None:
        response = encoding.MessageToPyValue(response_msg)
        if 'name' in response:
          log.UpdatedResource(response['name'], kind='Vertex AI index')
      return response_msg

  def Run(self, args):
    return self._Run(args, constants.GA_VERSION)


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
class UpdateV1Beta1(UpdateV1):
  """Update an Vertex AI index.

  ## EXAMPLES

  To update index `123` under project `example` in region `us-central1`, run:

    $ {command} --display-name=new-name
    --metadata-file=path/to/your/metadata.json --project=example
    --region=us-central1
  """

  def Run(self, args):
    return self._Run(args, constants.BETA_VERSION)




# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implementation of managed-folders add-iam-policy-binding command."""


from googlecloudsdk.api_lib.storage import api_factory
from googlecloudsdk.api_lib.storage import errors as api_errors
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.iam import iam_util
from googlecloudsdk.command_lib.storage import errors_util
from googlecloudsdk.command_lib.storage import iam_command_util
from googlecloudsdk.command_lib.storage import storage_url
from googlecloudsdk.command_lib.storage.tasks import set_iam_policy_task


@base.UniverseCompatible
class AddIamPolicyBinding(base.Command):
  """Add an IAM policy binding to a managed folder."""

  hints = base.CommandHint(read_only=False)

  detailed_help = {
      'DESCRIPTION': """
      Add an IAM policy binding to a managed folder. For more information, see [Cloud
      Identity and Access
      Management](https://cloud.google.com/storage/docs/access-control/iam).
      """,
      'EXAMPLES': """
      To grant a single role to a single principal for a managed folder `managed-folder` in a bucket `bucket`:

        $ {command} gs://bucket/managed-folder --member=user:john.doe@example.com --role=roles/storage.objectCreator

      To make objects in `gs://bucket/managed-folder` publicly readable:

        $ {command} gs://bucket/managed-folder --member=allUsers --role=roles/storage.objectViewer

      To specify a custom role for a principal on `gs://bucket/managed-folder`:

        $ {command} gs://bucket/managed-folder --member=user:john.doe@example.com --role=roles/customRoleName
      """,
  }

  @staticmethod
  def Args(parser):
    parser.add_argument(
        'url', help='URL of the managed folder to add IAM policy binding to.'
    )
    iam_util.AddArgsForAddIamPolicyBinding(parser, add_condition=True)

  def Run(self, args):
    url = storage_url.storage_url_from_string(args.url)
    errors_util.raise_error_if_not_gcs_folder_type(args.command_path, url)

    api_client = api_factory.get_api(url.scheme)
    messages = apis.GetMessagesModule('storage', 'v1')

    try:
      policy = api_client.get_managed_folder_iam_policy(
          url.bucket_name, url.resource_name
      )
    except api_errors.NotFoundError:
      api_client.create_managed_folder(url.bucket_name, url.resource_name)
      policy = messages.Policy()

    return iam_command_util.add_iam_binding_to_resource(
        args,
        url,
        messages,
        policy,
        set_iam_policy_task.SetManagedFolderIamPolicyTask,
    )



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Helper for processing API responses."""


from googlecloudsdk.core import log


def GetFieldAndLogUnreachable(message, attribute):
  """Response callback to log unreachable while generating fields of the message."""
  if message.unreachable:
    log.warning(
        'The following instances were unreachable: {}.'
        .format(', '.join(message.unreachable)))
  return getattr(message, attribute)


def GetFieldAndLogUnreachableInstancePartitions(message, attribute):
  """Response callback to log unreachable while generating fields of the message."""
  warning_text = 'The following instance partitions were unreachable: {}.'
  if hasattr(message, 'unreachable') and message.unreachable:
    # if the `message` is `ListInstancePartitionsResponse`.
    log.warning(warning_text.format(', '.join(message.unreachable)))
  elif (
      hasattr(message, 'unreachableInstancePartitions')
      and message.unreachableInstancePartitions
  ):
    # If the `message` is `ListInstancePartitionOperationsResponse`.
    log.warning(
        warning_text.format(', '.join(message.unreachableInstancePartitions))
    )
  return getattr(message, attribute)



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The gen-config command."""
# TODO(b/172812887) - Delete this command entirely; it's PY2 only.


import io
import os
import shutil
import tempfile

from gae_ext_runtime import ext_runtime
from googlecloudsdk.api_lib.app import yaml_parsing
from googlecloudsdk.api_lib.app.runtimes import fingerprinter
from googlecloudsdk.appengine.api import appinfo
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.app import deployables
from googlecloudsdk.command_lib.app import exceptions
from googlecloudsdk.command_lib.app import output_helpers
from googlecloudsdk.core import log
from googlecloudsdk.core.console import console_io
from googlecloudsdk.core.util import files
from ruamel import yaml
import six


_DEPRECATION_MSG = """\
This command is deprecated and will soon be removed.

{fingerprinting}

To create a custom runtime, please follow the instructions at
https://cloud.google.com/appengine/docs/flexible/custom-runtimes/
""".format(fingerprinting=deployables.FINGERPRINTING_WARNING)


# TODO(b/28509217): Move back into command class when `preview` is gone.
def _Args(parser):
  """Add arguments for `gcloud app gen-config`."""
  parser.add_argument(
      'source_dir',
      nargs='?',
      help='The source directory to fingerprint.',
      default=files.GetCWD())
  parser.add_argument(
      '--config',
      default=None,
      help=('The yaml file defining the service configuration.  This is '
            'normally one of the generated files, but when generating a '
            'custom runtime there can be an app.yaml containing parameters.'))

  rt_list = [r for r in appinfo.GetAllRuntimes() if r not in ['vm', 'custom']]
  parser.add_argument(
      '--runtime',
      default=None,
      help=('Generate config files for a given runtime. Can be used in '
            'conjunction with --custom. Allowed runtimes are: ' +
            ', '.join(rt_list) + '.'))
  parser.add_argument(
      '--custom',
      action='store_true',
      default=False,
      help=('If true, generate config files for a custom runtime.  This '
            'will produce a Dockerfile, a .dockerignore file and an app.yaml '
            '(possibly other files as well, depending on the runtime).'))


def _Run(args):
  """Run the `gcloud app gen-config` command."""
  if args.config:
    # If the user has specified an config file, use that.
    config_filename = args.config
  else:
    # Otherwise, check for an app.yaml in the source directory.
    config_filename = os.path.join(args.source_dir, 'app.yaml')
    if not os.path.exists(config_filename):
      config_filename = None

  # If there was an config file either specified by the user or in the source
  # directory, load it.
  if config_filename:
    try:
      myi = yaml_parsing.ServiceYamlInfo.FromFile(config_filename)
      config = myi.parsed
    except IOError as ex:
      log.error('Unable to open %s: %s', config_filename, ex)
      return
  else:
    config = None

  fingerprinter.GenerateConfigs(
      args.source_dir,
      ext_runtime.Params(appinfo=config, custom=args.custom,
                         runtime=args.runtime),
      config_filename)

  # If the user has a config file, make sure that they're using a custom
  # runtime.
  if config and args.custom and config.GetEffectiveRuntime() != 'custom':
    alter = console_io.PromptContinue(
        default=False,
        message=output_helpers.RUNTIME_MISMATCH_MSG.format(config_filename),
        prompt_string='Would you like to update it now?')
    if alter:
      _AlterRuntime(config_filename, 'custom')
      log.status.Print('[{0}] has been updated.'.format(config_filename))
    else:
      log.status.Print('Please update [{0}] manually by changing the runtime '
                       'field to custom.'.format(config_filename))


@base.Deprecate(is_removed=False, warning=_DEPRECATION_MSG)
@base.ReleaseTracks(base.ReleaseTrack.BETA)
class GenConfig(base.Command):
  """Generate missing configuration files for a source directory."""

  @staticmethod
  def Args(parser):
    _Args(parser)

  def Run(self, args):
    if six.PY3:
      raise exceptions.NotSupportedPy3Exception(
          'This command does not support python3.')
    _Run(args)


def _AlterRuntime(config_filename, runtime):
  try:
    # 0. Take backup
    with tempfile.NamedTemporaryFile(prefix='app.yaml.') as f:
      backup_fname = f.name
    log.status.Print(
        'Copying original config [{0}] to backup location [{1}].'.format(
            config_filename, backup_fname))
    shutil.copyfile(config_filename, backup_fname)
    # 1. Open and parse file using ruamel
    with files.FileReader(config_filename) as yaml_file:
      encoding = yaml_file.encoding
      config = yaml.load(yaml_file, yaml.RoundTripLoader)
    # 2. Alter the ruamel in-memory object representing the yaml file
    config['runtime'] = runtime
    # 3. Create an in-memory file buffer and write yaml file to it
    raw_buf = io.BytesIO()
    tmp_yaml_buf = io.TextIOWrapper(raw_buf, encoding)
    yaml.dump(config, tmp_yaml_buf, Dumper=yaml.RoundTripDumper)
    # 4. Overwrite the original app.yaml
    with files.BinaryFileWriter(config_filename) as yaml_file:
      tmp_yaml_buf.seek(0)
      yaml_file.write(raw_buf.getvalue())
  except Exception as e:
    raise fingerprinter.AlterConfigFileError(e)

_DETAILED_HELP = {
    'DESCRIPTION': """\
    This command generates all relevant config files (app.yaml, Dockerfile and a
    build Dockerfile) for your application in the current directory or emits an
    error message if the source directory contents are not recognized.
    """,
    'EXAMPLES': """\
    To generate configs for the current directory:

      $ {command}

    To generate configs for ~/my_app:

      $ {command} ~/my_app
    """
}

GenConfig.detailed_help = _DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to register App Hub application resources."""

from typing import Any
from argparse import Namespace
from argparse import ArgumentParser
import textwrap
from googlecloudsdk.api_lib.design_center import utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.design_center import flags
from googlecloudsdk.core import log


_DETAILED_HELP = {
    'DESCRIPTION': (
        'Register App Hub application resources.'
    ),
    'EXAMPLES': textwrap.dedent("""\
      To register App Hub application resources in project `my-project` location `us-central1`, with apphub application `projects/my-project/locations/us-central1/applications/my-application` and tfstate signed GCS URI `my-tfstate-signed-gcs-uri`, run:

          $ {command} --project=my-project --location=us-central1 --apphub-application=projects/my-project/locations/us-central1/applications/my-application --tfstate-signed-gcs-uri=my-tfstate-signed-gcs-uri
          """),
    'API REFERENCE': textwrap.dedent("""\
        This command uses the designcenter/v1 API. The full documentation for
        this API can be found at:
        http://cloud.google.com/application-design-center/docs
        """),
}


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.GA)
@base.Hidden
@base.UniverseCompatible
class RegisterAppHubApplicationResources(base.Command):
  """Registration of App Hub application resources.

  Attributes:
    detailed_help: A dictionary with detailed help information for the command.
  """

  detailed_help = _DETAILED_HELP

  @staticmethod
  def Args(parser: ArgumentParser)-> None:
    """Register flags for this command."""
    flags.AddRegisterAppHubApplicationResourcesFlags(parser)

  def Run(self, args: Namespace) -> Any:
    """This is what gets called when the user runs this command."""
    client = utils.GetClientInstance(self.ReleaseTrack())
    messages = utils.GetMessagesModule(self.ReleaseTrack())

    register_request_payload = messages.RegisterAppHubApplicationResourcesRequest(
        tfstateContent=args.tfstate_content,
        tfstateSignedGcsUri=args.tfstate_signed_gcs_uri,
        apphubApplication=args.apphub_application,
    )
    request = messages.DesigncenterProjectsLocationsSpacesRegisterAppHubApplicationResourcesRequest(
        parent=f'projects/{args.project}/locations/{args.location}',
        registerAppHubApplicationResourcesRequest=register_request_payload,
    )

    op = client.projects_locations_spaces.RegisterAppHubApplicationResources(
        request
    )

    if args.async_:
      log.status.Print('Registration started asynchronously.')
      return op

    response = utils.WaitForOperationWithEmbeddedResult(
        client,
        op,
        'Waiting for registration to complete',
        max_wait_sec=1800,
        release_track=self.ReleaseTrack(),
    )

    log.status.Print(
        'Registration completed for AppHub application'
    )

    if response.get('registeredResources'):
      log.status.Print('Successfully registered resources:')
      for res in response['registeredResources']:
        log.status.Print('- {0}'.format(res))

    if response.get('unregisteredResources'):
      log.warning('Failed to register resources:')
      for failed_res in response['unregisteredResources']:
        log.warning(
            '- Resource: %s, Error: %s',
            failed_res['deployedResource']['resourceId'],
            failed_res['reason'],
        )

    return response



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to list persistent resources in Vertex AI."""


from googlecloudsdk.api_lib.ai.persistent_resources import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.ai import constants
from googlecloudsdk.command_lib.ai import endpoint_util
from googlecloudsdk.command_lib.ai import flags
from googlecloudsdk.command_lib.ai import region_util
from googlecloudsdk.command_lib.ai.persistent_resources import persistent_resource_util
from googlecloudsdk.command_lib.ai.persistent_resources import validation


@base.ReleaseTracks(base.ReleaseTrack.GA)
class ListGA(base.ListCommand):
  """Lists the active persistent resources.

  ## EXAMPLES

  To list the persistent resources of project ``example'' in region
  ``us-central1'', run:

    $ {command} --project=example --region=us-central1
  """
  _api_version = constants.GA_VERSION

  @classmethod
  def Args(cls, parser):
    """Method called by Calliope to set up arguments for this command.

    Args:
      parser: A argparse.Parser to register accepted arguments in command input.
    """
    flags.AddRegionResourceArg(
        parser,
        'to list persistent resources',
        prompt_func=region_util.GetPromptForRegionFunc(
            constants.SUPPORTED_TRAINING_REGIONS))
    flags.AddUriFlags(
        parser,
        collection=persistent_resource_util.PERSISTENT_RESOURCE_COLLECTION,
        api_version=constants.AI_PLATFORM_API_VERSION[cls._api_version])

  def Run(self, args):
    """Executes the list command.

    Args:
      args: an argparse.Namespace, it contains all arguments that this command
        was invoked with.

    Returns:
      The list of resources
    """
    region_ref = args.CONCEPTS.region.Parse()
    region = region_ref.AsDict()['locationsId']
    validation.ValidateRegion(region)

    with endpoint_util.AiplatformEndpointOverrides(
        version=self._api_version, region=region):
      response = client.PersistentResourcesClient(
          version=self._api_version).List(
              region=region_ref.RelativeName())
      # Filter out resources with a raySpec present to hide ray clusters from
      # the user. In both the error and success case, the response will be a
      # generator object `YieldFromList`
      response = [resource for resource in response if (
          (resource.resourceRuntimeSpec is None) or
          (resource.resourceRuntimeSpec.raySpec is None))]
      return response


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class ListPreGA(ListGA):
  """Lists the active persistent resources.

  ## EXAMPLES

  To list the persistent resources of project ``example'' in region
  ``us-central1'', run:

    $ {command} --project=example --region=us-central1
  """
  _api_version = constants.BETA_VERSION



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for deleting network firewall policy associations."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute.network_firewall_policies import client
from googlecloudsdk.api_lib.compute.network_firewall_policies import region_client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.network_firewall_policies import flags


class Delete(base.CreateCommand):
  r"""Delete a new association between a firewall policy and an network or folder resource.

  *{command}* is used to delete network firewall policy associations. An
  network firewall policy is a set of rules that controls access to various
  resources.
  """
  NEWORK_FIREWALL_POLICY_ARG = None

  @classmethod
  def Args(cls, parser):
    cls.NETWORK_FIREWALL_POLICY_ARG = (
        flags.NetworkFirewallPolicyAssociationArgument(
            required=True, operation='delete'))
    cls.NETWORK_FIREWALL_POLICY_ARG.AddArgument(parser, operation_type='delete')
    flags.AddArgsDeleteAssociation(parser)
    parser.display_info.AddCacheUpdater(flags.NetworkFirewallPoliciesCompleter)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    ref = self.NETWORK_FIREWALL_POLICY_ARG.ResolveAsResource(
        args, holder.resources)

    network_firewall_policy = client.NetworkFirewallPolicy(
        ref, compute_client=holder.client)
    if hasattr(ref, 'region'):
      network_firewall_policy = region_client.RegionNetworkFirewallPolicy(
          ref, compute_client=holder.client)

    return network_firewall_policy.DeleteAssociation(
        firewall_policy=args.firewall_policy,
        name=args.name,
        only_generate_request=False)


Delete.detailed_help = {
    'EXAMPLES':
        """\
    To delete an association from a global network firewall policy with NAME
    ``my-policy'' and association name ``my-association'', run:

      $ {command}
          --firewall-policy=my-policy
          --name=my-association
          --global-firewall-policy

    To delete an association from a regional network firewall policy with NAME
    ``my-policy'' in region ``region-a'' and association name
    ``my-association'', run:

      $ {command}
          --firewall-policy=my-policy
          --name=my-association
          --firewall-policy-region=region-a
    """,
}



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for defining Tag resource manager arguments on a parser."""


import re

from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.iam import util as iam_api
from googlecloudsdk.api_lib.resource_manager import tags
from googlecloudsdk.api_lib.resource_manager.exceptions import ResourceManagerError
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.iam import iam_util
from googlecloudsdk.command_lib.resource_manager import endpoint_utils as endpoints
from googlecloudsdk.core import exceptions as core_exceptions


class InvalidInputError(ResourceManagerError):
  """Exception for invalid input."""

TAG_KEYS = 'tagKeys'
TAG_VALUES = 'tagValues'
TAG_BINDINGS = 'tagBindings'

GetRequests = {
    TAG_KEYS: tags.TagMessages().CloudresourcemanagerTagKeysGetRequest,
    TAG_VALUES: tags.TagMessages().CloudresourcemanagerTagValuesGetRequest,
}

GetByNamespacedNameRequests = {
    TAG_KEYS: (
        tags.TagMessages().CloudresourcemanagerTagKeysGetNamespacedRequest
    ),
    TAG_VALUES: (
        tags.TagMessages().CloudresourcemanagerTagValuesGetNamespacedRequest
    ),
}

ListRequests = {
    TAG_KEYS: tags.TagMessages().CloudresourcemanagerTagKeysListRequest,
    TAG_VALUES: tags.TagMessages().CloudresourcemanagerTagValuesListRequest,
    TAG_BINDINGS: tags.TagMessages().CloudresourcemanagerTagBindingsListRequest,
}

Services = {
    TAG_KEYS: tags.TagKeysService,
    TAG_VALUES: tags.TagValuesService,
    TAG_BINDINGS: tags.TagBindingsService,
}

MAX_TAG_KEYS = 1000
MAX_TAG_VALUES = 1000


def GetResource(resource_name, resource_type):
  """Gets the resource from the resource name.

  Args:
    resource_name: The resource name, such as tagKeys/{tag_key_id} or
      tagValues/{tag_value_id}
    resource_type: the type of the resource i.e: tag_utils.TAG_KEYS,
      tag_utils.TAG_VALUES. Used to determine which service to use and which GET
      request to construct

  Returns:
    resource
  """

  # TagKeys and TagValues GET require the global CRM API endpoint.
  with endpoints.CrmEndpointOverrides('global'):
    service = Services[resource_type]()
    req = GetRequests[resource_type](name=resource_name)
    response = service.Get(req)

    return response


def GetNamespacedResource(namespaced_name, resource_type):
  """Gets the resource from the namespaced name.

  Args:
    namespaced_name: The namespaced name of the resource such as
      {parent_id}/{tag_key_short_name} or
      {parent_id}/{tag_key_short_name}/{tag_value_short_name}
    resource_type: the type of the resource i.e: tag_utils.TAG_KEYS,
      tag_utils.TAG_VALUES. Used to determine which service to use and which GET
      request to construct

  Returns:
    resource
  """

  # GetNamespacedTag(Key|Value) call requires the global CRM API endpoint.
  with endpoints.CrmEndpointOverrides('global'):
    service = Services[resource_type]()
    req = GetByNamespacedNameRequests[resource_type](name=namespaced_name)
    response = service.GetNamespaced(req)

    return response


def GetTagValueParent(parent):
  if parent.find('tagValues/') == 0 or parent.find('tagKeys/') == 0:
    tag_value_parent = parent
  else:
    if parent.count('/') == 1:
      tag_value_parent = GetNamespacedResource(parent, TAG_KEYS).name
    else:
      tag_value_parent = GetNamespacedResource(parent, TAG_VALUES).name
  return tag_value_parent


def ProjectNameToBinding(project_name, tag_value, location=None):
  """Returns the binding name given a project name and tag value.

  Requires binding list permission.

  Args:
    project_name: project name provided, fully qualified resource name
    tag_value: tag value to match the binding name to
    location: region or zone

  Returns:
    binding_name

  Raises:
    InvalidInputError: project not found
  """
  service = Services[TAG_BINDINGS]()
  with endpoints.CrmEndpointOverrides(location):
    req = ListRequests[TAG_BINDINGS](parent=project_name)

    response = service.List(req)

    for bn in response.tagBindings:
      if bn.tagValue == tag_value:
        return bn.name

    raise InvalidInputError(
        'Binding not found for parent [{}], tagValue [{}]'.format(
            project_name, tag_value
        )
    )


def GetCanonicalResourceName(resource_name, location, release_track):
  """Returns the correct canonical name for the given resource.

  Args:
    resource_name: name of the resource
    location: location in which the resource lives
    release_track: release stage of current endpoint

  Returns:
    resource_name: either the original resource name, or correct canonical name

  Raises:
    InvalidArgumentException: if the location is not specified
  """
  service_account_resource_name_pattern = (
      r'iam.*/projects/[^/]+/serviceAccounts/([^/]+)'
  )
  service_account_search = re.search(
      service_account_resource_name_pattern, resource_name
  )

  if service_account_search:
    service_account_name = service_account_search.group(1)
    # Call IAM's service account describe API to get the service account's
    # unique id.
    if re.search('.*@.*.gserviceaccount.com', service_account_name):
      resource_name = resource_name.replace(
          'serviceAccounts/%s' % service_account_name,
          'serviceAccounts/%s'
          % _GetServiceAccountUniqueId(service_account_name),
      )
    return resource_name

  # [a-z]([-a-z0-9]*[a-z0-9] is the instance name regex, as per
  # https://cloud.google.com/compute/docs/reference/rest/v1/instances

  gce_compute_instance_name_pattern = (
      r'compute.googleapis.com/projects/([^/]+)/.*instances/([^/]+)'
  )
  gce_search = re.search(gce_compute_instance_name_pattern, resource_name)

  if gce_search:
    if not location:
      raise exceptions.InvalidArgumentException(
          '--location',
          (
              'Please specify an appropriate cloud location with the --location'
              ' flag.'
          ),
      )
    project_identifier, instance_identifier = gce_search.group(
        1
    ), gce_search.group(2)
    # call compute instance's describe api to get canonical resource name
    # use that instead of the instance name that's in the parent
    if re.search('([a-z]([-a-z0-9]*[a-z0-9])?)', instance_identifier):
      resource_name = resource_name.replace(
          'instances/%s' % instance_identifier,
          'instances/%s'
          % _GetGceInstanceCanonicalName(
              project_identifier, instance_identifier, location, release_track
          ),
      )
  return resource_name


def _GetServiceAccountUniqueId(service_account_email):
  """Returns the unique id for the given service account email.

  Args:
    service_account_email: email of the service account.

  Returns:
    The unique id of the service account.
  """
  client, messages = iam_api.GetClientAndMessages()
  try:
    res = client.projects_serviceAccounts.Get(
        messages.IamProjectsServiceAccountsGetRequest(
            name=iam_util.EmailToAccountResourceName(service_account_email)))
    return str(res.uniqueId)
  except apitools_exceptions.HttpError as e:
    raise exceptions.HttpException(e)


def _GetGceInstanceCanonicalName(
    project_identifier, instance_identifier, location, release_track
):
  """Returns the correct canonical name for the given gce compute instance.

  Args:
    project_identifier: project number of the compute instance
    instance_identifier: name of the instance
    location: location in which the resource lives
    release_track: release stage of current endpoint

  Returns:
    instance_id: returns the canonical instance id
  """
  compute_holder = base_classes.ComputeApiHolder(release_track)
  client = compute_holder.client
  request = (
      client.apitools_client.instances,
      'Get',
      client.messages.ComputeInstancesGetRequest(
          instance=instance_identifier,
          project=project_identifier,
          zone=location,
      ),
  )
  errors_to_collect = []
  instances = client.MakeRequests(
      [request], errors_to_collect=errors_to_collect
  )
  if errors_to_collect:
    raise core_exceptions.MultiError(errors_to_collect)
  return str(instances[0].id)


def ParseTagGroup(args, original):
  """Parses the tag keys and values into a map to be used for update."""

  if args.IsSpecified('clear_tags'):
    return {}

  tags_map_to_update = {}

  if args.IsSpecified('update_tags'):
    tags_dict = args.update_tags

    tag_group = ExtractExistingTags(original, {})

    for tag_key, tag_value in tags_dict.items():
      tag_group[tag_key] = tag_value

    tags_map_to_update = tag_group

  if args.IsSpecified('remove_tags'):
    tags_list = args.remove_tags
    tag_group = tags_map_to_update

    tag_group = ExtractExistingTags(original, tag_group)

    for tag_key in tags_list:
      if '=' in tag_key:
        raise exceptions.InvalidArgumentException(
            '--remove_tags',
            'Please specify the tag key only in the namespaced format. i.e'
            ' --remove-tags=foo/bar,foo2/bar2',
        )
      if tag_key in tag_group:
        unused_removed_tag = tag_group.pop(tag_key)
        # remove the tag from the original tags
    tags_map_to_update.update(tag_group)

  if args.IsSpecified('replace_tags'):
    tags_dict = args.replace_tags
    tag_group = {}
    for tag_key, tag_value in tags_dict.items():
      tag_group[tag_key] = tag_value
    tags_map_to_update = tag_group

  return tags_map_to_update


def ExtractExistingTags(original, tag_group):
  """Extracts the existing tags from the original tags."""
  if original.tags:
    additional_property = original.tags.additionalProperties
    for property_item in additional_property:
      tag_group[property_item.key] = property_item.value
  return tag_group



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for deleting configuration for application awareness on interconnect."""

from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute.interconnects import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.interconnects import flags


DETAILED_HELP = {
    'DESCRIPTION': """\
        *{command}* is used to delete all configuration state for
        application awareness on interconnect.

        For an example, refer to the *EXAMPLES* section below.
        """,
    'EXAMPLES': """\
        To delete application awareness configuration for an interconnect
        example-interconnect, run:

          $ {command} example-interconnect
        """,
}


@base.UniverseCompatible
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.GA,
    base.ReleaseTrack.PREVIEW,
)
class Delete(base.UpdateCommand):
  """Delete application awareness configuration of a Compute Engine interconnect.

  *{command}* allows the user to delete application awareness configuration data
  associated with
  Compute Engine interconnect in a project.
  """

  INTERCONNECT_ARG = None

  @classmethod
  def Args(cls, parser):
    cls.INTERCONNECT_ARG = flags.InterconnectArgument()
    cls.INTERCONNECT_ARG.AddArgument(parser, operation_type='patch')

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    ref = self.INTERCONNECT_ARG.ResolveAsResource(args, holder.resources)
    interconnect = client.Interconnect(ref, compute_client=holder.client)
    messages = holder.client.messages

    application_awareness = messages.InterconnectApplicationAwareInterconnect()
    cleared_fields = [
        'applicationAwareInterconnect.shapeAveragePercentages',
        'applicationAwareInterconnect.bandwidthPercentagePolicy',
        'applicationAwareInterconnect.strictPriorityPolicy',
    ]
    application_awareness.strictPriorityPolicy = None
    application_awareness.bandwidthPercentagePolicy = None
    application_awareness.profileDescription = ''
    application_awareness.shapeAveragePercentages = []

    return interconnect.Patch(
        description=None,
        interconnect_type=None,
        requested_link_count=None,
        link_type=None,
        admin_enabled=None,
        noc_contact_email=None,
        location=None,
        labels=None,
        label_fingerprint=None,
        macsec_enabled=None,
        macsec=None,
        aai_enabled=False,
        application_aware_interconnect=application_awareness,
        cleared_fields=cleared_fields,
    )


Delete.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for updating HTTP health checks."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.health_checks import exceptions
from googlecloudsdk.command_lib.compute.http_health_checks import flags
from googlecloudsdk.core import log

THRESHOLD_UPPER_BOUND = 10
THRESHOLD_LOWER_BOUND = 1
TIMEOUT_UPPER_BOUND_SEC = 300
TIMEOUT_LOWER_BOUND_SEC = 1
CHECK_INTERVAL_UPPER_BOUND_SEC = 300
CHECK_INTERVAL_LOWER_BOUND_SEC = 1


class Update(base.UpdateCommand):
  """Update a legacy HTTP health check.

  *{command}* is used to update an existing legacy HTTP health check. Only
  arguments passed in will be updated on the health check. Other
  attributes will remain unaffected.
  """

  HTTP_HEALTH_CHECKS_ARG = None

  @classmethod
  def Args(cls, parser):
    cls.HTTP_HEALTH_CHECKS_ARG = flags.HttpHealthCheckArgument()
    cls.HTTP_HEALTH_CHECKS_ARG.AddArgument(parser, operation_type='update')

    parser.add_argument(
        '--host',
        help="""\
        The value of the host header used in this HTTP health check request.
        By default, this is empty and Compute Engine automatically sets
        the host header in health requests to the same external IP address as
        the forwarding rule associated with the target pool. Setting this to
        an empty string will clear any existing host value.
        """)

    parser.add_argument(
        '--port',
        type=int,
        help="""\
        The TCP port number that this health check monitors.
        """)

    parser.add_argument(
        '--request-path',
        help="""\
        The request path that this health check monitors. For example,
        ``/healthcheck''.
        """)

    parser.add_argument(
        '--check-interval',
        type=arg_parsers.Duration(),
        help="""\
        How often to perform a health check for an instance. For example,
        specifying ``10s'' will run the check every 10 seconds.
        See $ gcloud topic datetimes for information on duration formats.
        """)

    parser.add_argument(
        '--timeout',
        type=arg_parsers.Duration(),
        help="""\
        If Compute Engine doesn't receive an HTTP 200 response from the
        instance by the time specified by the value of this flag, the health
        check request is considered a failure. For example, specifying ``10s''
        will cause the check to wait for 10 seconds before considering the
        request a failure.  Valid units for this flag are ``s'' for seconds and
        ``m'' for minutes.
        """)

    parser.add_argument(
        '--unhealthy-threshold',
        type=int,
        help="""\
        The number of consecutive health check failures before a healthy
        instance is marked as unhealthy.
        """)

    parser.add_argument(
        '--healthy-threshold',
        type=int,
        help="""\
        The number of consecutive successful health checks before an
        unhealthy instance is marked as healthy.
        """)

    parser.add_argument(
        '--description',
        help=('A textual description for the HTTP health check. Pass in an '
              'empty string to unset.'))

  @property
  def service(self):
    return self.compute.httpHealthChecks

  @property
  def resource_type(self):
    return 'httpHealthChecks'

  def CreateReference(self, resources, args):
    return self.HTTP_HEALTH_CHECKS_ARG.ResolveAsResource(
        args, resources)

  def GetGetRequest(self, client, http_health_check_ref):
    """Returns a request for fetching the existing HTTP health check."""
    return (client.apitools_client.httpHealthChecks,
            'Get',
            client.messages.ComputeHttpHealthChecksGetRequest(
                httpHealthCheck=http_health_check_ref.Name(),
                project=http_health_check_ref.project))

  def GetSetRequest(self, client, http_health_check_ref, replacement):
    """Returns a request for updated the HTTP health check."""
    return (client.apitools_client.httpHealthChecks,
            'Update',
            client.messages.ComputeHttpHealthChecksUpdateRequest(
                httpHealthCheck=http_health_check_ref.Name(),
                httpHealthCheckResource=replacement,
                project=http_health_check_ref.project))

  def Modify(self, client, args, existing_check):
    """Returns a modified HttpHealthCheck message."""
    # Description and Host are the only attributes that can be cleared by
    # passing in an empty string (but we don't want to set it to an empty
    # string).
    if args.description:
      description = args.description
    elif args.description is None:
      description = existing_check.description
    else:
      description = None

    if args.host:
      host = args.host
    elif args.host is None:
      host = existing_check.host
    else:
      host = None

    new_health_check = client.messages.HttpHealthCheck(
        name=existing_check.name,
        host=host,
        port=args.port or existing_check.port,
        description=description,
        requestPath=args.request_path or existing_check.requestPath,
        checkIntervalSec=(args.check_interval or
                          existing_check.checkIntervalSec),
        timeoutSec=args.timeout or existing_check.timeoutSec,
        healthyThreshold=(args.healthy_threshold or
                          existing_check.healthyThreshold),
        unhealthyThreshold=(args.unhealthy_threshold or
                            existing_check.unhealthyThreshold),
    )
    return new_health_check

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client

    if (args.check_interval is not None
        and (args.check_interval < CHECK_INTERVAL_LOWER_BOUND_SEC
             or args.check_interval > CHECK_INTERVAL_UPPER_BOUND_SEC)):
      raise exceptions.ArgumentError(
          '[--check-interval] must not be less than {0} second or greater '
          'than {1} seconds; received [{2}] seconds.'.format(
              CHECK_INTERVAL_LOWER_BOUND_SEC, CHECK_INTERVAL_UPPER_BOUND_SEC,
              args.check_interval))

    if (args.timeout is not None
        and (args.timeout < TIMEOUT_LOWER_BOUND_SEC
             or args.timeout > TIMEOUT_UPPER_BOUND_SEC)):
      raise exceptions.ArgumentError(
          '[--timeout] must not be less than {0} second or greater than {1} '
          'seconds; received: [{2}] seconds.'.format(
              TIMEOUT_LOWER_BOUND_SEC, TIMEOUT_UPPER_BOUND_SEC, args.timeout))

    if (args.healthy_threshold is not None
        and (args.healthy_threshold < THRESHOLD_LOWER_BOUND
             or args.healthy_threshold > THRESHOLD_UPPER_BOUND)):
      raise exceptions.ArgumentError(
          '[--healthy-threshold] must be an integer between {0} and {1}, '
          'inclusive; received: [{2}].'.format(THRESHOLD_LOWER_BOUND,
                                               THRESHOLD_UPPER_BOUND,
                                               args.healthy_threshold))

    if (args.unhealthy_threshold is not None
        and (args.unhealthy_threshold < THRESHOLD_LOWER_BOUND
             or args.unhealthy_threshold > THRESHOLD_UPPER_BOUND)):
      raise exceptions.ArgumentError(
          '[--unhealthy-threshold] must be an integer between {0} and {1}, '
          'inclusive; received [{2}].'.format(THRESHOLD_LOWER_BOUND,
                                              THRESHOLD_UPPER_BOUND,
                                              args.unhealthy_threshold))

    args_unset = not (args.port
                      or args.request_path
                      or args.check_interval
                      or args.timeout
                      or args.healthy_threshold
                      or args.unhealthy_threshold)
    if args.description is None and args.host is None and args_unset:
      raise exceptions.ArgumentError('At least one property must be modified.')

    http_health_check_ref = self.CreateReference(holder.resources, args)
    get_request = self.GetGetRequest(client, http_health_check_ref)

    objects = client.MakeRequests([get_request])

    new_object = self.Modify(client, args, objects[0])

    # If existing object is equal to the proposed object or if
    # Modify() returns None, then there is no work to be done, so we
    # print the resource and return.
    if objects[0] == new_object:
      log.status.Print(
          'No change requested; skipping update for [{0}].'.format(
              objects[0].name))
      return objects

    return client.MakeRequests(
        [self.GetSetRequest(client, http_health_check_ref, new_object)])



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""api registry mcp server disable command."""


from googlecloudsdk.calliope import base
from googlecloudsdk.core import log


# TODO(b/321801975) make command public after preview.
@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class DisableAlpha(base.SilentCommand):
  """Disables MCP server for a given service in the current project."""

  @staticmethod
  def Args(parser):
    parser.add_argument(
        'service',
        help='The MCP server to disable.',
    )

  def Run(self, args):
    """Disables MCP server for a given service in the current project."""
    log.status.Print(
        'MCP server enablement is no longer required, and enabling the'
        ' underlying service is now sufficient. This command will be'
        ' deprecated and has no effect.'
    )


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.BETA)
class DisableBeta(base.SilentCommand):
  """Disables MCP server for a given service in the current project."""

  @staticmethod
  def Args(parser):
    parser.add_argument(
        'service',
        help='The MCP server to disable.',
    )

  def Run(self, args):
    """Disables MCP server for a given service in the current project."""
    log.status.Print(
        'MCP server enablement is no longer required, and enabling the'
        ' underlying service is now sufficient. This command will be'
        ' deprecated and has no effect.'
    )



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""API Library for gcloud cloudscheduler."""


from googlecloudsdk.api_lib.scheduler import cmek_config
from googlecloudsdk.api_lib.scheduler import jobs
from googlecloudsdk.api_lib.scheduler import locations
from googlecloudsdk.api_lib.scheduler import operations
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.calliope import base


API_NAME = 'cloudscheduler'
ALPHA_API_VERSION = 'v1alpha1'
BETA_API_VERSION = 'v1beta1'
GA_API_VERSION = 'v1'


class UnsupportedReleaseTrackError(Exception):
  """Raised when requesting an api for an unsupported release track."""


def ApiVersionFromReleaseTrack(release_track):
  if release_track == base.ReleaseTrack.ALPHA:
    return ALPHA_API_VERSION
  if release_track == base.ReleaseTrack.BETA:
    return BETA_API_VERSION
  if release_track == base.ReleaseTrack.GA:
    return GA_API_VERSION
  else:
    raise UnsupportedReleaseTrackError(release_track)


def GetApiAdapter(release_track, legacy_cron=False):
  if release_track == base.ReleaseTrack.ALPHA:
    return AlphaApiAdapter(legacy_cron=legacy_cron)
  elif release_track == base.ReleaseTrack.BETA:
    return BetaApiAdapter(legacy_cron=legacy_cron)
  elif release_track == base.ReleaseTrack.GA:
    return GaApiAdapter(legacy_cron=legacy_cron)
  else:
    raise UnsupportedReleaseTrackError(release_track)


class BaseApiAdapter(object):

  def __init__(self, api_version):
    self.client = apis.GetClientInstance(API_NAME, api_version)
    self.messages = self.client.MESSAGES_MODULE
    self.locations = locations.Locations(
        self.client.MESSAGES_MODULE, self.client.projects_locations
    )
    self.cmek_config = cmek_config.CmekConfig(
        self.client.MESSAGES_MODULE, self.client.projects_locations
    )


class AlphaApiAdapter(BaseApiAdapter):

  def __init__(self, legacy_cron=False):
    super(AlphaApiAdapter, self).__init__(ALPHA_API_VERSION)
    self.jobs = jobs.BaseJobs(
        self.client.MESSAGES_MODULE,
        self.client.projects_locations_jobs,
        legacy_cron=legacy_cron,
    )


class BetaApiAdapter(BaseApiAdapter):

  def __init__(self, legacy_cron=False):
    super(BetaApiAdapter, self).__init__(BETA_API_VERSION)
    self.jobs = jobs.BaseJobs(
        self.client.MESSAGES_MODULE,
        self.client.projects_locations_jobs,
        legacy_cron=legacy_cron,
    )


class GaApiAdapter(BaseApiAdapter):

  def __init__(self, legacy_cron=False):
    super(GaApiAdapter, self).__init__(GA_API_VERSION)
    self.jobs = jobs.BaseJobs(
        self.client.MESSAGES_MODULE,
        self.client.projects_locations_jobs,
        legacy_cron=legacy_cron,
    )
    self.operations = operations.Operations(
        self.client.MESSAGES_MODULE, self.client.projects_locations_operations
    )



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to list unregistered workloads in the Project/Location."""

from googlecloudsdk.api_lib.apphub import discovered_workloads as apis
from googlecloudsdk.api_lib.apphub import utils as api_lib_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.apphub import flags


_DETAILED_HELP = {
    'DESCRIPTION': '{description}',
    'EXAMPLES': """ \
        To list all unregistered DiscoveredWorkloads in location `us-east1`,
        run:

          $ {command} --location=us-east1
        """,
}

_FORMAT = """
  table(
    name.scope("discoveredWorkloads"):label=ID,
    workloadReference,
    workloadProperties
  )
"""


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class FindUnregistered(base.ListCommand):
  """List unregistered Apphub discovered workloads."""

  detailed_help = _DETAILED_HELP

  @staticmethod
  def Args(parser):
    flags.AddFindUnregisteredWorkloadFlags(parser)
    parser.display_info.AddFormat(_FORMAT)
    parser.display_info.AddUriFunc(
        api_lib_utils.MakeGetUriFunc(
            'apphub.projects.locations.discoveredWorkloads'
        )
    )

  def Run(self, args):
    """Run the find unregistered workload command."""
    client = apis.DiscoveredWorkloadsClient()
    location_ref = args.CONCEPTS.location.Parse()
    return client.FindUnregistered(
        limit=args.limit,
        page_size=args.page_size,
        parent=location_ref.RelativeName(),
    )



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags for backup-dr restore compute commands."""


from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.command_lib.backupdr import util
from googlecloudsdk.command_lib.util.apis import arg_utils


def AddNameArg(parser, required=True):
  parser.add_argument(
      '--name',
      type=str,
      required=required,
      help='Name of the restored Compute Instance.',
  )


def AddTargetZoneArg(parser, required=True):
  parser.add_argument(
      '--target-zone',
      type=str,
      required=required,
      help='Zone where the target instance is restored.',
  )


def AddTargetProjectArg(parser, required=True):
  parser.add_argument(
      '--target-project',
      type=str,
      required=required,
      help='Project where the restore should happen.',
  )


def AddNetworkInterfaceArg(parser, required=True):
  """Network interface of the restored resource."""
  network_tier_options = ('NETWORK_TIER_UNSPECIFIED', 'STANDARD', 'PREMIUM')
  stack_type_options = ('STACK_TYPE_UNSPECIFIED', 'IPV4_ONLY', 'IPV4_IPV6')
  nic_type_options = ('NIC_TYPE_UNSPECIFIED', 'VIRTIO_NET', 'GVNIC')

  network_tier_validator = util.GetOneOfValidator(
      'network-tier', network_tier_options
  )
  stack_type_validator = util.GetOneOfValidator(
      'stack-type', stack_type_options
  )
  nic_type_validator = util.GetOneOfValidator('nic-type', nic_type_options)
  network_interface_spec = {
      'network': str,
      'subnet': str,
      'private-network-ip': str,
      'internal-ipv6-address': str,
      'internal-ipv6-prefix-length': int,
      'address': str,
      'external-ipv6-address': str,
      'external-ipv6-prefix-length': int,
      'network-tier': network_tier_validator,
      'aliases': str,
      'stack-type': stack_type_validator,
      'queue-count': int,
      'nic-type': nic_type_validator,
      'network-attachment': str,
  }
  parser.add_argument(
      '--network-interface',
      required=required,
      type=arg_parsers.ArgDict(spec=network_interface_spec),
      action='append',
      metavar='PROPERTY=VALUE',
      help=(
          'Adds a network interface to the instance. This flag can be repeated'
          ' to specify multiple network interfaces. The following keys are'
          ' allowed: network, subnet, private-network-ip,'
          ' internal-ipv6-address, internal-ipv6-prefix-length, address,'
          ' external-ipv6-address, external-ipv6-prefix-length,'
          ' network-tier, aliases,'
          ' stack-type, queue-count, nic-type, network-attachment'
      ),
  )


def AddServiceAccountArg(parser, required=True):
  """Service account used to restore."""
  parser.add_argument(
      '--service-account',
      type=str,
      required=required,
      help=(
          'A service account is an identity attached to the instance. Its'
          ' access tokens can be accessed through the instance metadata server'
          ' and are used to authenticate applications on the instance. The'
          ' account can be set using an email address corresponding to the'
          ' required service account. If not provided, the instance will use'
          " the project's default service account."
      ),
  )


def AddScopesArg(parser, required=True):
  """Scopes for the service account used to restore."""
  scopes_group = parser.add_mutually_exclusive_group()

  scopes_group.add_argument(
      '--scopes',
      type=arg_parsers.ArgList(str),
      required=required,
      metavar='SCOPE',
      help=(
          'If not provided, the instance will be assigned the default scopes,'
          ' described below. However, if neither --scopes nor --no-scopes are'
          ' specified and the project has no default service account, then the'
          ' instance will be created with no scopes. Note that the level of'
          ' access that a service account has is determined by a combination of'
          ' access scopes and IAM roles so you must configure both access'
          ' scopes and IAM roles for the service account to work properly.'
          ' SCOPE can be either the full URI of the scope or an alias. Default'
          ' scopes are assigned to all instances. Available aliases are:'
          ' https://cloud.google.com/sdk/gcloud/reference/compute/instances/create#--scopes'
      ),
  )

  scopes_group.add_argument(
      '--no-scopes',
      action='store_true',
      help='Create the instance with no scopes.',
  )


def AddCreateDiskArg(parser, required=True):
  """Attaches persistent disks to the instances."""

  disk_spec = {
      'name': str,
      'replica-zones': arg_parsers.ArgList(str, custom_delim_char=' '),
      'device-name': str,
      'kms-key': str,
  }
  parser.add_argument(
      '--create-disk',
      required=required,
      type=arg_parsers.ArgDict(spec=disk_spec),
      action='append',
      metavar='PROPERTY=VALUE',
      help="""
          Creates and attaches persistent disks to the instances.

          *name*: Specifies the name of the disk.

          *replica-zones*: Required for each regional disk associated with the
          instance. Specify the URLs of the zones where the disk should be
          replicated to. You must provide exactly two replica zones, and one
          zone must be the same as the instance zone.

          *device-name*: Device name of the disk from the source instance.

          *kms-key*: The Cloud KMS key resource name to be used for encryption.
          If supplied, it will encrypt the disk with the given key,
          if not supplied, the restored disk will use the encryption
          of the disk from the source instance.
          The `device-name` field must be provided for `kms-key` to take effect.
          Format:
          'projects/{project}/locations/{location}/keyRings/{ring}/cryptoKeys/{key}'
          """,
  )


def AddDescriptionArg(parser, required=True):
  """Description of the restored instance."""
  parser.add_argument(
      '--description',
      type=str,
      required=required,
      help='Specifies a textual description of the restored instance.',
  )


def AddMetadataArg(parser, required=True):
  """Metadata to be made available to the guest operating system."""
  helptext = """\
      Metadata to be made available to the guest operating system
      running on the instances. Each metadata entry is a key/value
      pair separated by an equals sign. Each metadata key must be unique
      and have a max of 128 bytes in length. Each value must have a max of
      256 KB in length. Multiple arguments can be
      passed to this flag, e.g.,
      ``--metadata key-1=value-1,key-2=value-2,key-3=value-3''.
      The combined total size for all metadata entries is 512 KB.

      In images that have Compute Engine tools installed on them,
      such as the
      link:https://cloud.google.com/compute/docs/images[official images],
      the following metadata keys have special meanings:

      *startup-script*::: Specifies a script that will be executed
      by the instances once they start running.

      *startup-script-url*::: Same as ``startup-script'' except that
      the script contents are pulled from a publicly-accessible
      location on the web.


      For startup scripts on Windows instances, the following metadata keys
      have special meanings:
      ``windows-startup-script-url'',
      ``windows-startup-script-cmd'', ``windows-startup-script-bat'',
      ``windows-startup-script-ps1'', ``sysprep-specialize-script-url'',
      ``sysprep-specialize-script-cmd'', ``sysprep-specialize-script-bat'',
      and ``sysprep-specialize-script-ps1''. For more information, see
      [Running startup scripts](https://cloud.google.com/compute/docs/startupscript).
      """
  parser.add_argument(
      '--metadata',
      required=required,
      type=arg_parsers.ArgDict(min_length=1),
      default={},
      metavar='KEY=VALUE',
      help=helptext,
      action=arg_parsers.StoreOnceAction,
  )


def AddLabelsArg(parser, required=True):
  """Labels to be added to the instance."""
  helptext = """\
      List of label KEY=VALUE pairs to add.

      Keys must start with a lowercase character and contain only hyphens (-),
      underscores (_), lowercase characters, and numbers. Values must contain
      only hyphens (-), underscores (_), lowercase characters, and numbers.
      """
  parser.add_argument(
      '--labels',
      required=required,
      type=arg_parsers.ArgDict(),
      default={},
      metavar='KEY=VALUE',
      help=helptext,
  )


def AddTagsArg(parser, required=True):
  """Tags to be added to the instance."""
  helptext = """\
      Specifies a list of tags to apply to the instance. These tags allow
      network firewall rules and routes to be applied to specified VM instances.
      See gcloud_compute_firewall-rules_create(1) for more details.
      """
  parser.add_argument(
      '--tags',
      required=required,
      type=arg_parsers.ArgList(min_length=1),
      metavar='TAG',
      help=helptext,
  )


def AddMachineTypeArg(parser, required=True):
  """Machine type used for the restored instance."""
  helptext = """\
      Specifies the machine type used for the restored instance. To get a list
      of available machine types, run 'gcloud compute machine-types list'. If
      unspecified, the default type will be based on the source instance.

      This can either be the fully qualified path or the name.
      For example:
      * ``--machine-type=projects/my-project/zones/us-central1-a/machineTypes/n1-standard-1''
      * ``--machine-type=n1-standard-1''
      """
  parser.add_argument(
      '--machine-type',
      type=str,
      required=required,
      help=helptext,
  )


def AddHostnameArg(parser, required=True):
  """Hostname of the restore instance."""
  helptext = """\
      Specify the hostname of the restore instance to be created. The specified
      hostname must be RFC1035 compliant. If hostname is not specified, the
      default hostname is [INSTANCE_NAME].c.[TARGET_PROJECT_ID].internal when using the
      global DNS, and [INSTANCE_NAME].[ZONE].c.[TARGET_PROJECT_ID].internal when using
      zonal DNS.
      """
  parser.add_argument(
      '--hostname',
      type=str,
      required=required,
      help=helptext,
  )


def AddEnableUefiNetworkingArg(parser, required=True):
  """Enable UEFI networking for the instance creation."""
  helptext = """\
      If set to true, enables UEFI networking for the instance creation.
      """
  parser.add_argument(
      '--enable-uefi-networking',
      action=arg_parsers.StoreTrueFalseAction,
      required=required,
      help=helptext,
  )


def AddThreadsPerCoreArg(parser, required=True):
  """The number of visible threads per physical core."""
  helptext = """
      The number of visible threads per physical core. To disable simultaneous
      multithreading (SMT) set this to 1. Valid values are: 1 or 2.

      For more information about configuring SMT, see:
      https://cloud.google.com/compute/docs/instances/configuring-simultaneous-multithreading.
    """
  parser.add_argument(
      '--threads-per-core',
      type=int,
      required=required,
      help=helptext,
  )


def AddVisibleCoreCountArg(parser, required=True):
  """The number of physical cores to expose to the instance's guest operating system."""
  helptext = """
      The number of physical cores to expose to the instance's guest operating
      system. The number of virtual CPUs visible to the instance's guest
      operating system is this number of cores multiplied by the instance's
      count of visible threads per physical core.
    """
  parser.add_argument(
      '--visible-core-count',
      type=int,
      required=required,
      help=helptext,
  )


def AddAcceleratorArg(parser, required=True):
  """Attaches accelerators (e.g. GPUs) to the instances."""
  helptext = """\
      Attaches accelerators (e.g. GPUs) to the instances.

      *type*::: The specific type (e.g. nvidia-tesla-k80 for nVidia Tesla K80)
      of accelerator to attach to the instances. Use 'gcloud compute
      accelerator-types list' to learn about all available accelerator types.

      *count*::: Number of accelerators to attach to each
      instance. The default value is 1.
      """
  parser.add_argument(
      '--accelerator',
      type=arg_parsers.ArgDict(
          spec={'type': str, 'count': int}, required_keys=['type']
      ),
      required=required,
      help=helptext,
  )


def AddMinCpuPlatform(parser, required=True):
  """Minimum CPU platform to be used for the instance."""
  helptext = """\
      When specified, the VM will be scheduled on host with specified CPU
      architecture or a newer one. To list available CPU platforms in given
      zone, run:

          $ gcloud compute zones describe ZONE --format="value(availableCpuPlatforms)"

      Default setting is "AUTOMATIC".

      CPU platform selection is available only in selected zones.

      You can find more information on-line:
      [](https://cloud.google.com/compute/docs/instances/specify-min-cpu-platform)
      """
  parser.add_argument(
      '--min-cpu-platform',
      type=str,
      required=required,
      metavar='PLATFORM',
      help=helptext,
  )


def AddMaintenancePolicyArg(parser, required=True):
  """Maintenance policy to be used for the instance."""
  helptext = """\
      Specifies the behavior of the VMs when their host machines undergo
      maintenance. The default is MIGRATE.
      For more information, see
      https://cloud.google.com/compute/docs/instances/host-maintenance-options.
      """
  parser.add_argument(
      '--maintenance-policy',
      choices={
          'MIGRATE': (
              'The instances should be migrated to a new host. This will'
              ' temporarily impact the performance of instances during a'
              ' migration event.'
          ),
          'TERMINATE': 'The instances should be terminated.',
      },
      type=arg_utils.ChoiceToEnumName,
      required=required,
      help=helptext,
  )


def AddPreemptibleArg(parser, required=True):
  """Preemptible state to be used for the instance."""
  helptext = """\
      If provided, instances will be preemptible and time-limited. Instances
      might be preempted to free up resources for standard VM instances,
      and will only be able to run for a limited amount of time. Preemptible
      instances can not be restarted and will not migrate.
      """
  parser.add_argument(
      '--preemptible',
      action=arg_parsers.StoreTrueFalseAction,
      required=required,
      help=helptext,
  )


def AddRestartOnFailureArg(parser, required=True):
  """Restart on failure state to be used for the instance."""
  helptext = """\
      The instances will be restarted if they are terminated by Compute Engine.
      This does not affect terminations performed by the user.
      """
  parser.add_argument(
      '--restart-on-failure',
      action=arg_parsers.StoreTrueFalseAction,
      required=required,
      help=helptext,
  )


def AddMinNodeCpuArg(parser, required=False):
  """Minimum Node CPUs to be used for the instance."""
  helptext = """\
      Minimum number of virtual CPUs this instance will consume when running on
      a sole-tenant node.
      """
  parser.add_argument(
      '--min-node-cpu',
      type=int,
      required=required,
      help=helptext,
  )


def AddProvisioningModelArg(parser, required=False):
  """Provisioning model to be used for the instance."""
  helptext = """\
      Specifies provisioning model, which determines price, obtainability,
      and runtime for the restored VM instance.
      """
  parser.add_argument(
      '--provisioning-model',
      choices={
          'SPOT': (
              'Spot VMs are spare capacity; Spot VMs are discounted '
              'to have much lower prices than standard VMs '
              'but have no guaranteed runtime. Spot VMs are the new version '
              'of preemptible VM instances, except Spot VMs do not have '
              'a 24-hour maximum runtime.'
          ),
          'STANDARD': (
              'Default. Standard provisioning model for VM instances, '
              'which has user-controlled runtime '
              'but no Spot discounts.'
          ),
      },
      type=arg_utils.ChoiceToEnumName,
      required=required,
      help=helptext,
  )


def AddInstanceTerminationActionArg(parser, required=False):
  """Termination action to be used for the instance."""
  helptext = """\
      Specifies the termination action that will be taken upon VM preemption
        (--provisioning-model=SPOT) or automatic instance
        termination (--max-run-duration or --termination-time).
      """
  parser.add_argument(
      '--instance-termination-action',
      choices={
          'STOP': (
              'Default only for Spot VMs. Stop the VM without preserving'
              ' memory. The VM can be restarted later.'
          ),
          'DELETE': 'Permanently delete the VM.',
      },
      type=arg_utils.ChoiceToEnumName,
      required=required,
      help=helptext,
  )


def AddLocalSsdRecoveryTimeoutArg(parser, required=False):
  """Local SSD recovery timeout to be used for the instance."""
  helptext = """\
      Specifies the maximum amount of time a Local SSD VM should wait while
      recovery of the Local SSD state is attempted. Its value should be in
      between 0 and 168 hours with hour granularity and the default value being 1
      hour.
      """
  parser.add_argument(
      '--local-ssd-recovery-timeout',
      type=arg_parsers.Duration(
          default_unit='h', lower_bound='0h', upper_bound='168h'
      ),
      required=required,
      help=helptext,
  )


def AddNodeAffinityFileArg(parser, required=False):
  """Node affinity file to be used for the instance."""
  helptext = """\
      The JSON/YAML file containing the configuration of desired nodes onto
      which this instance could be scheduled. These rules filter the nodes
      according to their node affinity labels. A node's affinity labels come
      from the node template of the group the node is in.

      The file should contain a list of a JSON/YAML objects. For an example,
      see https://cloud.google.com/compute/docs/nodes/provisioning-sole-tenant-vms#configure_node_affinity_labels.
      The following list describes the fields:

      *key*::: Corresponds to the node affinity label keys of
      the Node resource.
      *operator*::: Specifies the node selection type. Must be one of:
        `IN`: Requires Compute Engine to seek for matched nodes.
        `NOT_IN`: Requires Compute Engine to avoid certain nodes.
      *values*::: Optional. A list of values which correspond to the node
      affinity label values of the Node resource.
      """
  parser.add_argument(
      '--node-affinity-file',
      type=arg_parsers.YAMLFileContents(),
      required=required,
      help=helptext,
      hidden=True,
  )


def AddReservationArgs(parser, required=False):
  """Reservation affinity to be used for the instance."""
  reservation_group_helptext = """Specifies the reservation for the instance."""
  reservation_affinity_helptext = """\
      Specifies the reservation affinity of the instance.
      """
  reservation_help_text = """The name of the reservation, required when `--reservation-affinity=specific`."""

  group = parser.add_group(help=reservation_group_helptext)

  group.add_argument(
      '--reservation-affinity',
      choices={
          'any': 'Consume any available, matching reservation.',
          'none': 'Do not consume from any reserved capacity.',
          'specific': 'Must consume from a specific reservation.',
      },
      required=required,
      help=reservation_affinity_helptext,
  )

  group.add_argument(
      '--reservation',
      required=required,
      help=reservation_help_text,
  )


def AddEnableDisplayDeviceArg(parser, required=False):
  """Enable display device for the instance."""
  helptext = """\
      Enable a display device on the restored VM instances. Disabled by default.
      """
  parser.add_argument(
      '--enable-display-device',
      action=arg_parsers.StoreTrueFalseAction,
      required=required,
      help=helptext,
  )


def AddCanIpForwardArg(parser, required=False):
  """Enable can ip forward for the restored instance."""
  helptext = """\
      If provided, allows the restored instances to send and receive packets
      with non-matching destination or source IP addresses.
      """
  parser.add_argument(
      '--can-ip-forward',
      action=arg_parsers.StoreTrueFalseAction,
      required=required,
      help=helptext,
  )


def AddPrivateIpv6GoogleAccessArg(parser, required=False):
  """Enable private ipv6 google access for the  restored instance."""
  helptext = """\
      The private IPv6 Google access type for the restored VM.
      """
  enum_mappings = {
      'inherit-subnetwork': 'INHERIT_FROM_SUBNETWORK',
      'enable-bidirectional-access': 'ENABLE_BIDIRECTIONAL_ACCESS_TO_GOOGLE',
      'enable-outbound-vm-access': 'ENABLE_OUTBOUND_VM_ACCESS_TO_GOOGLE',
  }
  helptext += 'PRIVATE_IPV6_GOOGLE_ACCESS_TYPE must be one of: {}'.format(
      ', '.join(list(enum_mappings.keys()))
  )
  parser.add_argument(
      '--private-ipv6-google-access-type',
      type=util.EnumMapper(enum_mappings).Parse,
      required=required,
      help=helptext,
  )


def AddNetworkPerformanceConfigsArg(parser, required=False):
  """Enable network performance config for the restored instance."""
  helptext = """\
      Configures network performance settings for the restored instance.
      If this flag is not specified, the restored instance will be created
      with its source instance's network performance configuration.

      *total-egress-bandwidth-tier*::: Total egress bandwidth is the available
      outbound bandwidth from a VM, regardless of whether the traffic
      is going to internal IP or external IP destinations.
      The following tier values are allowed: [DEFAULT, TIER_1]
      """
  parser.add_argument(
      '--network-performance-configs',
      type=arg_parsers.ArgDict(
          spec={
              'total-egress-bandwidth-tier': str,
          },
      ),
      metavar='PROPERTY=VALUE',
      required=required,
      help=helptext,
  )


def AddConfidentialComputeArg(parser, required=False):
  """Enable confidential compute for the restored instance."""
  helptext = """\
      The restored instance boots with Confidential Computing enabled.
      Confidential Computing is based on Secure Encrypted Virtualization (SEV),
      an AMD virtualization feature for running confidential instances.
      """
  parser.add_argument(
      '--confidential-compute',
      action='store_true',
      required=required,
      help=helptext,
  )


def AddDeletionProtectionArg(parser, required=False):
  """Enable deletion protection for the restored instance."""
  helptext = """\
      Enables deletion protection for the restored instance.
      """
  parser.add_argument(
      '--deletion-protection',
      action=arg_parsers.StoreTrueFalseAction,
      required=required,
      help=helptext,
  )


def AddResourceManagerTagsArg(parser, required=False):
  """ResourceManagerTags to be added to the instance."""
  helptext = """\
      Specifies a list of resource manager tags to apply to the instance.
      """
  parser.add_argument(
      '--resource-manager-tags',
      type=arg_parsers.ArgDict(),
      metavar='KEY=VALUE',
      required=required,
      help=helptext,
  )


def AddResourcePoliciesArg(parser, required=False):
  """ResourcePolicies to be added to the instance."""
  helptext = """\
      A list of resource policy names to be added to the instance.
      The policies must exist in the same region as the instance.
      """
  parser.add_argument(
      '--resource-policies',
      type=arg_parsers.ArgList(min_length=1),
      metavar='RESOURCE_POLICY',
      required=required,
      help=helptext,
  )


def AddKeyRevocationActionTypeArg(parser, required=False):
  """KeyRevocationActionType to be added to the instance."""
  helptext = """\
      Specifies the behavior of the instance when the KMS key of one of its
      attached disks is revoked. The default is none. POLICY must be one of:
      * none
      No operation is performed.
      * stop
      The instance is stopped when the KMS key of one of its attached disks is
      revoked.
      """
  enum_mappings = {
      'none': 'NONE',
      'stop': 'STOP',
  }
  parser.add_argument(
      '--key-revocation-action-type',
      metavar='POLICY',
      type=util.EnumMapper(enum_mappings).Parse,
      required=required,
      help=helptext,
  )


def AddInstanceKmsKeyArg(parser, required=False):
  """InstanceKmsKey to be added to the instance."""
  helptext = """\
      The Cloud KMS (Key Management Service) cryptokey that will be used to
      protect the restored instance.

      Provide the full resource name of the cryptokey in the format:
      `projects/<project>/locations/<location>/keyRings/<key-ring>/cryptoKeys/<key>`
      """
  parser.add_argument(
      '--instance-kms-key',
      type=str,
      required=required,
      help=helptext,
  )


def AddClearEncryptionKeyArg(parser):
  """Clear encryption key override for the disk."""
  helptext = """\
      Clears the CMEK encryption of the restored disks and defaults to GMEK
      unless the kms-key is specified in the --create-disk flag.
      """
  parser.add_argument(
      '--clear-encryption-key',
      action='store_true',
      required=False,
      help=helptext,
  )
